# -*- python -*-
# $RCSfile: microstructurePage.py,v $
# $Revision: 1.59 $
# $Author: langer $
# $Date: 2004/10/04 18:50:03 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common import debug
from common import guitop
from common import microstructure
from common.IO import mainmenu
from common.IO import parameter
from common.IO import questioner
from common.IO import whoville
from common.IO.GUI import chooser
from common.IO.GUI import fileselector
from common.IO.GUI import oofGUI
from common.IO.GUI import parameterwidgets
from common.IO.GUI import whowidget
from gtk import *
from types import *

class MicrostructurePage(oofGUI.MainPage):
    def __init__(self):
        debug.mainthreadTest()

        self.suppress_grplist = 0

        oofGUI.MainPage.__init__(self, name="Microstructure", ordering=10,
                                 tip="Define Microstructure and Pixel Group objects.")
        vbox = GtkVBox(spacing=2)
        self.gtk.add(vbox)


        hbox = GtkHBox()
        vbox.add(hbox, expand=0, fill=0)
        centerbox = GtkHBox()
        hbox.add(centerbox, expand=1, fill=0)
        label=GtkLabel('Microstructure=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        self.mswidget = whowidget.WhoWidget(microstructure.microStructures,
                                            callback=self.msCB)
        centerbox.add(self.mswidget.gtk[0], expand=0, fill=0)
        
        hbox = GtkHBox()                # first row of ms buttons
        vbox.add(hbox, expand=0, fill=0)
        self.newbuttonbox = GtkHBox(homogeneous=1, spacing=3)
        hbox.add(self.newbuttonbox, expand=1, fill=0)

        self.newbutton = GtkButton('New...')
        self.newbutton.connect('clicked', self.newEmptyCB)
        guitop.top().tooltip(self.newbutton,
                             "Create a new microstructure that is NOT associated with images.")
        self.newbuttonbox.add(self.newbutton, expand=1, fill=1)
        
        # Other buttons can be added to the row of "New" buttons by
        # other modules.  When they're added, by addNewButton(), a
        # function can be specified for sensitizing the button.  This
        # is the list of those functions:
        self.sensitizeFns = []

        hbox = GtkHButtonBox()                # second row of ms buttons
        vbox.add(hbox, expand=0, fill=0)
        centerbox = GtkHBox(homogeneous=1, spacing=3)
        hbox.add(centerbox, expand=1, fill=0)

        self.renamebutton = GtkButton('Rename...')
        self.renamebutton.connect('clicked', self.renameMSCB)
        guitop.top().tooltip(self.renamebutton,
                             "Rename the current microstructure.")
        centerbox.add(self.renamebutton, expand=1, fill=1)

        self.copybutton = GtkButton('Copy...')
        self.copybutton.connect('clicked', self.copyMSCB)
        guitop.top().tooltip(self.copybutton,
                             "Copy the current microstructure.")        
        centerbox.add(self.copybutton, expand=1, fill=1)

        self.deletebutton = GtkButton('Delete')
        self.deletebutton.connect('clicked', self.deleteMSCB)
        guitop.top().tooltip(self.deletebutton,
                             "Delete the current microstructure.")
        centerbox.add(self.deletebutton, expand=1, fill=1)

        self.savebutton = GtkButton('Save...')
        self.savebutton.connect('clicked', self.saveMSCB)
        guitop.top().tooltip(self.savebutton,
                             "Save the current microstructure to a file.")
        centerbox.add(self.savebutton, expand=1, fill=1)

        pane = GtkHPaned()
        vbox.add(pane, expand=1, fill=1, padding=2)

        #######
        
        infoframe = GtkFrame('Microstructure Info')
        pane.pack1(infoframe)
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        infoframe.add(scroll)
        self.infoarea = GtkText()
        self.infoarea.set_word_wrap(TRUE)
        scroll.add_with_viewport(self.infoarea)
        
        ########
        
        groupframe = GtkFrame('Pixel Groups')
        pane.pack2(groupframe)
        hbox = GtkHBox()
        groupframe.add(hbox)
        vbox = GtkVBox(spacing=2)       # buttons on L side of pixel group list
        hbox.add(vbox, expand=0, fill=0, padding=2)
        frame = GtkFrame()              # frame for the list of groups
        frame.set_shadow_type(SHADOW_IN)
        hbox.add(frame)
        grparea = GtkVBox()
        frame.add(grparea)
        # only one of grplist and grpmsg is visible at a time
        self.grplist = chooser.ScrolledChooserListWidget( # list of pixel groups
            callback=self.listItemChosen)
        grparea.add(self.grplist.gtk)
        self.grpmsg = GtkLabel()        # helpful message when there are no grps
        grparea.add(self.grpmsg)

        self.newgroupbutton = GtkButton('New...')
        vbox.add(self.newgroupbutton, expand=0, fill=0)
        self.newgroupbutton.connect('clicked', self.newGroupButtonCB)
        guitop.top().tooltip(self.newgroupbutton,
                             "Create a new empty pixel group in the current microstructure.")

        self.renamegroupbutton = GtkButton('Rename...')
        vbox.add(self.renamegroupbutton, expand=0, fill=0)
        self.renamegroupbutton.connect('clicked', self.renameGroupButtonCB)
        guitop.top().tooltip(self.renamegroupbutton,
                             "Rename the selected pixel group.")

        self.copygroupbutton = GtkButton('Copy...')
        vbox.add(self.copygroupbutton, expand=0, fill=0)
        self.copygroupbutton.connect('clicked', self.copyGroupButtonCB)
        guitop.top().tooltip(self.copygroupbutton,
                             "Create a new group containing the same pixels as the selected group.")

        self.delgroupbutton = GtkButton('Delete')
        vbox.add(self.delgroupbutton, expand=0, fill=0)
        self.delgroupbutton.connect('clicked', self.deleteGroupButtonCB)
        guitop.top().tooltip(self.delgroupbutton,
                             "Delete the selected pixel group from the microstructure.")

        self.meshablebutton = GtkCheckButton('Meshable')
        vbox.add(self.meshablebutton, expand=0, fill=0)
        self.meshablesignal = self.meshablebutton.connect('clicked',
                                                          self.meshableGroupCB)
        guitop.top().tooltip(self.meshablebutton,
                             "Should adaptive meshes follow the boundaries of the selected pixel group?")


        # buttons on rhs of pixelgroup list
        vbox = GtkVBox(spacing=2)
        hbox.add(vbox, expand=0, fill=0, padding=2)

        self.addbutton = GtkButton('Add')
        vbox.add(self.addbutton, expand=0, fill=0)
        self.addbutton.connect('clicked', self.addPixelsCB)
        guitop.top().tooltip(self.addbutton,
                             "Add the currently selected pixels to the selected group.")

        self.removebutton = GtkButton('Remove')
        vbox.add(self.removebutton, expand=0, fill=0)
        self.removebutton.connect('clicked', self.removePixelsCB)
        guitop.top().tooltip(self.removebutton,
                             "Remove the currently selected pixels from the selected group.")

        self.clearbutton = GtkButton('Clear')
        vbox.add(self.clearbutton, expand=0, fill=0)
        self.clearbutton.connect('clicked', self.clearPixelsCB)
        guitop.top().tooltip(self.clearbutton,
                             "Reset the selected group by removing all the pixels from the group.")
        
        self.infobutton = GtkButton('Info')
        vbox.add(self.infobutton, expand=0, fill=0)
        self.infobutton.connect('clicked', self.queryPixelsCB)
        guitop.top().tooltip(self.infobutton,
                             "Display information about the selected group in the Messages window.")        

        self.sbcallbacks = [
            switchboard.requestCallback('new pixel group', self.newpixgrp),
            switchboard.requestCallback('destroy pixel group', self.destpixgrp),
            switchboard.requestCallback('changed pixel group', self.destpixgrp),
            switchboard.requestCallback(('new who', 'Microstructure'),
                                        self.newwhoMS),
            switchboard.requestCallback(('new who', 'Image'),
                                        self.newwhoImage),
            switchboard.requestCallback(('rename who', 'Image'),
                                        self.displayMSInfo),
            switchboard.requestCallback('remove who', self.removewho),
            switchboard.requestCallback('renamed pixel group',
                                        self.renamepixgrp),
            switchboard.requestCallback('pixel selection changed',
                                        self.selectionchanged),
            switchboard.requestCallback('images changed in microstructure',
                                        self.displayMSInfo)
            ]
        self.sensitize()
        self.rebuild_grouplist()
        self.displayMSInfo()

    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.rebuild_grouplist()        # hides either grplist or grpmsg.
        
    def currentMSName(self):
        return self.mswidget.get_value()

    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            try:
                ctxt = microstructure.microStructures[msname]
            except KeyError:
                return None
            else:
                return ctxt.getObject()

    def currentGroupName(self):
        return self.grplist.get_value()

    def currentGroup(self):
        name = self.currentGroupName()
        if name:
            ms = self.currentMS()
            if ms is not None:
                return ms.findGroup(name)

    def rebuild_grouplist(self):
        debug.mainthreadTest()
        ms = self.currentMS()
        if ms is not None:
            if ms.nGroups() > 0:
                grpnames = ms.groupNames()
                dispnames = grpnames[:]
                for i in range(len(grpnames)):
                    grpname = grpnames[i]
                    grp = ms.findGroup(grpname)
                    if len(grp) == 1:
                        dispnames[i] += " (1 pixel"
                    else:
                        dispnames[i] += " (%d pixels" % len(grp)
                    if grp.is_meshable():
                        dispnames[i] += ", meshable)"
                    else:
                        dispnames[i] += ")"
                self.grplist.update(grpnames, dispnames)
                self.grplist.show()
                self.grpmsg.hide()
            else:
                self.grpmsg.set_text('No pixel groups defined!')
                self.grplist.hide()
                self.grpmsg.show()
        else:                           # ms is None
            self.grpmsg.set_text('No Microstructure defined!')
            self.grplist.hide()
            self.grpmsg.show()

    def addNewButton(self, gtkobj, sensitizefn):
        self.newbuttonbox.add(gtkobj, expand=1, fill=1)
        gtkobj.show()
        if sensitizefn is not None:
            self.sensitizeFns.append(sensitizefn)

    def displayMSInfo(self, *args):
        # *args is given because this is used as a switchboard
        # callback for ('rename who', 'Image') as well as being called
        # directly.  This indicates either slovenliness or efficiency
        # on the part of the programmer.
        debug.mainthreadTest()
        ms = self.currentMS()
        self.infoarea.backward_delete(self.infoarea.get_length())
        if ms is not None:
            size = ms.sizeInPixels()
            self.infoarea.insert_defaults('Pixel size: %dx%d\n' %
                                          (size.x, size.y))
            size = ms.size()
            self.infoarea.insert_defaults('Physical size: %gx%g\n' %
                                          (size.x, size.y))
            imagenames = ms.imageNames()
            if imagenames:
                self.infoarea.insert_defaults('Images:\n')
                for name in imagenames:
                    self.infoarea.insert_defaults('  ' + name + '\n')
                                          

    def setMeshableButton(self):
        debug.mainthreadTest()
        grp = self.currentGroup()
        self.meshablebutton.signal_handler_block(self.meshablesignal)
        if grp is not None:
            self.meshablebutton.set_active(grp.is_meshable())
        else:
            self.meshablebutton.set_active(0)
        self.meshablebutton.signal_handler_unblock(self.meshablesignal)
            
    def sensitize(self):
        debug.mainthreadTest()
        imageclass = whoville.getClass('Image')
        ms = self.currentMS()
        ms_defined = ms is not None
        self.renamebutton.set_sensitive(ms_defined)
        self.copybutton.set_sensitive(ms_defined)
        self.deletebutton.set_sensitive(ms_defined)
        self.savebutton.set_sensitive(ms_defined)

        grp = self.currentGroup()
        grp_selected = grp is not None
        self.newgroupbutton.set_sensitive(ms_defined)
        self.renamegroupbutton.set_sensitive(grp_selected)
        self.copygroupbutton.set_sensitive(grp_selected)
        self.delgroupbutton.set_sensitive(grp_selected)
        self.meshablebutton.set_sensitive(grp_selected)
        self.infobutton.set_sensitive(grp_selected)

        pixelsselected = ms and ms.pixelselection.size() > 0
        self.addbutton.set_sensitive(grp_selected and pixelsselected)
        self.removebutton.set_sensitive(grp_selected and pixelsselected)
        self.clearbutton.set_sensitive(grp_selected and len(grp)>0)

        for fn in self.sensitizeFns:
            fn()

    def selectionchanged(self):         # switchboard 'pixel selection changed'
        self.sensitize()

    ###############################

    def renameMSCB(self, button):
        menuitem = mainmenu.OOF.Microstructure.Rename
        namearg = menuitem.get_arg('name')
        namearg.value = self.currentMSName()
        if parameterwidgets.getParameters(
            namearg,
            title='Rename Microstructure '+self.currentMSName()):
            menuitem.callWithDefaults(microstructure=self.currentMSName())

    def copyMSCB(self, button):
        menuitem = mainmenu.OOF.Microstructure.Copy
        namearg = menuitem.get_arg('name')
        if parameterwidgets.getParameters(namearg, title='Copy microstructure'):
            menuitem.callWithDefaults(microstructure=self.currentMSName())

    def deleteMSCB(self, button):
        if questioner.questioner("Really delete %s?" % self.currentMSName(),
                                 "No", default="Yes") == "Yes":
            mainmenu.OOF.Microstructure.Delete(
                microstructure=self.currentMSName())

    def listItemChosen(self, name):
        self.sensitize()
        
    def newpixgrp(self, group):  # switchboard callback 'new pixel group'
##        if group.microstructure() is self.currentMS():
        self.rebuild_grouplist()
        # This makes newly added pixel group automatically selected.
        self.grplist.set_selection(group.name())
        self.sensitize()

    def destpixgrp(self, group): #'destroy pixel group' or 'changed pixel group'
##        if group.microstructure() is self.currentMS():
        self.rebuild_grouplist()
        self.sensitize()
    
    def newwhoMS(self, whoname): # switchboard ('new who', 'Microstructure')
        self.mswidget.set_value(whoname) # whoname is a list of strings
        self.rebuild_grouplist()
        self.displayMSInfo()
        self.sensitize()

    def newwhoImage(self, whoname):     # switchboard ('new who', 'Image')
        self.sensitize()

    def removewho(self, whoclassname, whoname): # switchboard 'remove who'
        if whoclassname == 'Microstructure':
            self.rebuild_grouplist()
            self.displayMSInfo()
            self.sensitize()
        elif whoclassname == 'Image':
            self.sensitize()

    def msCB(self, *args):              # whowidget callback
        self.rebuild_grouplist()
        self.displayMSInfo()
        self.sensitize()

    def newEmptyCB(self, *args):
        menuitem = mainmenu.OOF.Microstructure.New
        if parameterwidgets.getParameters(title='Create Microstructure',
                                          *menuitem.params):
            menuitem.callWithDefaults()

    def newGroupButtonCB(self, button):
        menuitem = mainmenu.OOF.PixelGroup.New
        nameparam = menuitem.get_arg('name')
        if parameterwidgets.getParameters(nameparam,
                                          title='Create new pixel group'):
            menuitem.callWithDefaults(microstructure=self.currentMSName())

    def renameGroupButtonCB(self, button):
        menuitem = mainmenu.OOF.PixelGroup.Rename
        nameparam = menuitem.get_arg('new_name')
        nameparam.value = self.currentGroupName()
        if parameterwidgets.getParameters(
            nameparam,
            title = 'Rename pixelgroup ' + self.currentGroupName()):
            menuitem.callWithDefaults(
                microstructure=self.currentMSName(),
                group=self.currentGroupName())

    def renamepixgrp(self, group, oldname, newname):
        # switchboard 'rename pixel group'
        ms = self.currentMS()
        if ms and group.name() in ms.groupNames():
            # Ensure that the new pixel group is selected.
            self.rebuild_grouplist()
            self.grplist.set_selection(newname)
        
    def deleteGroupButtonCB(self, button):
        if questioner.questioner("Really delete %s?" % self.currentGroupName(),
                                 "No", default="Yes") == "Yes":
            mainmenu.OOF.PixelGroup.Delete(microstructure=self.currentMSName(),
                                           group=self.currentGroupName())

    def copyGroupButtonCB(self, button):
        menuitem = mainmenu.OOF.PixelGroup.Copy
        nameparam = menuitem.get_arg('name')
##         nameparam.value = self.currentGroup().name()+'_copy'
        if parameterwidgets.getParameters(
            nameparam,
            title='Rename group '+self.currentGroup().name()):
            menuitem.callWithDefaults(microstructure=self.currentMSName(),
                                      group=self.currentGroupName())

    def meshableGroupCB(self, button):
        self.meshablebutton.signal_handler_block(self.meshablesignal)
        mainmenu.OOF.PixelGroup.Meshable(microstructure=self.currentMSName(),
                                group=self.currentGroupName(),
                                meshable=self.meshablebutton.active)
        self.meshablebutton.signal_handler_unblock(self.meshablesignal)

    def addPixelsCB(self, button):
        mainmenu.OOF.PixelGroup.AddSelection(microstructure=self.currentMSName(),
                                    group=self.currentGroupName())

    def removePixelsCB(self, button):
        mainmenu.OOF.PixelGroup.RemoveSelection(microstructure=self.currentMSName(),
                                       group=self.currentGroupName())

    def clearPixelsCB(self, button):
        mainmenu.OOF.PixelGroup.Clear(microstructure=self.currentMSName(),
                             group=self.currentGroupName())

    def queryPixelsCB(self, button):
        mainmenu.OOF.PixelGroup.Query(microstructure=self.currentMSName(),
                             group=self.currentGroupName())

    def saveMSCB(self, button):
        menuitem = mainmenu.OOF.File.Save.Microstructure
        msname = self.currentMSName()
        file = fileselector.fileSelector(
            ident='SaveMicrostructureFromPage',
            title='Save Microstructure "%s"' % msname,
            params=[menuitem.get_arg('format')])
        if file is not None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                menuitem.callWithDefaults(filename=file, mode=mode,
                                          microstructure=msname)
                
mp = MicrostructurePage()

#######################################

def _save_microstructure(menuitem):
    if len(microstructure.microStructures) > 0:
        file = fileselector.fileSelector(
            ident='SaveMicrostructureFromMenu',
            title='Save Microstructure',
            params=[menuitem.get_arg('microstructure'),
                    menuitem.get_arg('format')]
            )
        if file is not None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                menuitem.callWithDefaults(filename=file, mode=mode)
    else:
        reporter.report("No microstructures?")

mainmenu.OOF.File.Save.Microstructure.add_gui_callback(_save_microstructure)

########################################

# Add a button to the row of "New..." buttons
def addNewButton(gtkobj, sensitizefn=None):
    mp.addNewButton(gtkobj, sensitizefn)
