# -*- python -*-
# $RCSfile: oofGUI.py,v $
# $Revision: 1.37 $
# $Author: langer $
# $Date: 2004/10/04 18:50:03 $


# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import debug
from common import guitop
from common import utils
from common.IO import gfxmanager
from common.IO import mainmenu
from common.IO import oofmenu
from common.IO.GUI import chooser
from common.IO.GUI import gfxmenu
from common.IO.GUI import quit
from common.IO.GUI import widgetscope
from common.IO import reporter
import common.quit
from gtk import *

try:
    import GTK
except ImportError:
    import GTK_
    GTK = GTK_

allPages = {}                           # dictionary of pages keyed by name
pagenames = []                          # ordered list of pages

class oofGUI(widgetscope.WidgetScope):
    def __init__(self):
        debug.mainthreadTest()
        widgetscope.WidgetScope.__init__(self, None)
        self.gtk = GtkWindow(WINDOW_TOPLEVEL)
        self.gtk.set_title('OOF2')
        self.gtk.connect("destroy", self.destroyCB)
        self.gtk.connect("delete-event", self.deleteEventCB)
        guitop.setTop(self)

        self.tooltips = GtkTooltips()

        self.mainbox = GtkVBox()
        self.gtk.add(self.mainbox)

        self.menubar = GtkMenuBar()
        self.mainbox.add(self.menubar, expand=0, fill=0)
        accelgrp = GtkAccelGroup()
        self.gtk.add_accel_group(accelgrp)

        self.mainmenu = mainmenu.OOF
        self.oofmenu = gfxmenu.gtkOOFMenuBar(self.mainmenu, bar=self.menubar,
                                             accelgroup=accelgrp)

        self.pageChooserFrame = GtkFrame()
        self.pageChooserFrame.set_shadow_type(SHADOW_IN)
        self.mainbox.add(self.pageChooserFrame, expand=0, fill=0, padding=2)
        # The extra vbox is to put a little extra space around the
        # widgets in the frame.
        vbox = GtkVBox()
        self.pageChooserFrame.add(vbox)
        chooserBox = GtkHBox()
        vbox.add(chooserBox, padding=2)


        centerbox = GtkHBox()
        chooserBox.add(centerbox, expand=1, fill=0)
        label=GtkLabel('Task: ')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        self.prevPageButton = GtkButton()
        centerbox.add(self.prevPageButton, expand=0, fill=0,
                                padding=3)
        self.prevPageButton.add(GtkArrow(at=ARROW_LEFT))
        self.prevPageButton.connect('clicked', self.prevPageCB)
        self.pageChooser = chooser.ChooserWidget([],
                                                 callback=self.pageChooserCB)
        centerbox.add(self.pageChooser.gtk, expand=0, fill=0)
        self.currentPageName = None

        self.nextPageButton = GtkButton()
        centerbox.add(self.nextPageButton, expand=0, fill=0,
                                padding=3)
        self.nextPageButton.add(GtkArrow(at=ARROW_RIGHT))
        self.nextPageButton.connect('clicked', self.nextPageCB)

        # Add a GUI callback to the "OOF2" windows item.
        oof2_item = self.mainmenu.Windows.OOF2
        oof2_item.add_gui_callback(self.menu_raise)

       # Frame around main pages.  GUI pages are added and removed
       # from it by installPage().
        self.pageframe = GtkFrame()
        self.pageframe.set_usize(MainPage.initial_width,
                                 MainPage.initial_height)
        self.pageframe.set_shadow_type(SHADOW_IN)
        self.mainbox.add(self.pageframe)

        # Add pages that may have been created before the main GUI was built.
        for pagename, i in zip(pagenames, range(len(allPages))):
            self.addPage(allPages[pagename], i)

    def tooltip(self, widget, tip):
        debug.mainthreadTest()
        self.tooltips.set_tip(widget, tip)

    def installPage(self, pagename):
        # Actually install a page in the page frame.
        if self.currentPageName is not None:
            self.pageframe.remove(allPages[self.currentPageName].gtk)
        self.currentPageName = pagename
        self.pageframe.add(allPages[self.currentPageName].gtk)
        self.sensitize()

    def show(self, messages=[]):
        debug.mainthreadTest()
        if self.currentPageName is None:
            self.installPage(pagenames[0])
            self.pageChooser.set_state(self.currentPageName)

        # don't use self.gtk.show_all(), since there may be page
        # components that shouldn't yet be shown.
        self.menubar.show_all()
        self.pageChooserFrame.show_all()
        self.pageframe.show()
        self.mainbox.show()
        for page in allPages.values():
            page.show()
        self.gtk.show()

        for m in messages:
            reporter.report(m)
        
    def addPage(self, page, position):
        pagetips = {}
        for pg in allPages.values():
            if pg.tip:
                pagetips[pg.name]=pg.tip
        self.pageChooser.update(pagenames, pagetips)
        self.sensitize()

    def pageChooserCB(self, widget, pagename):
        self.installPage(pagename)

    def nextPageCB(self, button):
        which = pagenames.index(self.currentPageName)
        self.installPage(pagenames[which+1])
        self.pageChooser.set_state(self.currentPageName)

    def prevPageCB(self, button):
        which = pagenames.index(self.currentPageName)
        self.installPage(pagenames[which-1])        
        self.pageChooser.set_state(self.currentPageName)

    def sensitize(self):
        # If currentPageName is None, then the GUI hasn't been shown yet.
        if self.currentPageName is not None:
            which = pagenames.index(self.currentPageName)
            self.nextPageButton.set_sensitive(which != len(pagenames)-1)
            self.prevPageButton.set_sensitive(which != 0)

            if which < len(pagenames)-1:
                self.tooltip(self.nextPageButton, "Go to the %s page" %
                             allPages[pagenames[which+1]].name)
            if which > 0:
                self.tooltip(self.prevPageButton, "Go to the %s page" %
                             allPages[pagenames[which-1]].name)

    def deleteEventCB(self, *args):
        if quit.quitQuery():            # do you really want to quit?
            common.quit.cleanup()
            return 0                    # will call destroyCB
        return 1                        # don't actually destroy the window
    def destroyCB(self, gtk):
        self.gtk = None
        guitop.setTop(None)
    def destroy(self):
        debug.mainthreadTest()
        if self.gtk:
            self.gtk.destroy()

    def stop(self):
        mainquit()

    # GUI callback for "OOF2" entry in Windows menu.  Normally,
    # the "subwindow" class provides this functionality, but the main
    # window is not a subwindow.
    def menu_raise(self, menuitem):
        debug.mainthreadTest()
        self.gtk.get_window()._raise()
        

    
####################################################

# Pages added to the main OOF window must be subclasses of MainPage.
# The subclass's __init__ should call MainPage.__init__ and install
# all of its widgets into self.gtk, which is a GtkFrame constructed by
# MainPage.__init__.  A single instance of the subclass should be
# created.

class MainPage(widgetscope.WidgetScope):
    # TODO LATER: Set initial sizes in some more user-friendly way.
    initial_width = 550
    initial_height = 350
    def __init__(self, name, ordering, tip=None):
        debug.mainthreadTest()
        widgetscope.WidgetScope.__init__(self, parent=gui)
        self.name = name
        self.ordering = ordering
        self.tip = tip
        self.gtk = GtkFrame()
        self.gtk.set_shadow_type(SHADOW_NONE)
        # Insert the page in the proper spot
        for i in range(len(pagenames)):
            if self.ordering < allPages[pagenames[i]].ordering: # page goes here
                pagenames.insert(i, name)
                if guitop.top():        # gui may not be constructed yet!
                    guitop.top().addPage(self, i)
                break
        else:                           # page goes at the end
            pagenames.append(name)
            if guitop.top():            # gui may not be constructed yet!
                guitop.top().addPage(self, len(pagenames))
        allPages[name] = self

    # MainPage.show() should be redefined in subclasses that don't
    # always want to show all their widgets.
    def show(self):
        self.gtk.show_all()
    def hide(self):
        self.gtk.hide()
        

gui = oofGUI()


def start(messages=[]):
    debug.mainthreadTest()
    gui.show(messages)
    guitop.setMainLoop(1)
    mainloop()
    guitop.setMainLoop(0)

