// -*- C++ -*-
// $RCSfile: oofcanvas.h,v $
// $Revision: 1.48 $
// $Author: haan $
// $Date: 2004/06/17 21:36:02 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef OOFCANVAS_H
#define OOFCANVAS_H

class OOFCanvas;
class OOFCanvasLayer;

#include <Python.h>
#include "common/coord.h"
#include "common/IO/GUI/rubberband.h"
#include <gtk-canvas.h>
#include <gtk/gtk.h>
#include <gdk_imlib.h>
#include <vector>

class StringImage;
class CRectangle;

class OOFCanvas {
private:
  static bool initialized;
  bool isnew;
  bool empty;
  bool mousedown;
  Coord last_mouse;
  Coord mousedown_point;
  double affineflip[6];
  PyObject *mouse_callback;
  PyObject *config_callback;
  guint mouse_handler_id, config_handler_id;
  RubberBand *rubberband;
  OOFCanvasLayer *underlayer;
  GtkAdjustment *hadj;
  GtkAdjustment *vadj;
  RubberBand *make_rubberband(const Coord&);
protected:
  GtkWidget *canvas;
  double pixels_per_unit;	// zoom factor
  GtkCanvasGroup *root;
  std::vector<OOFCanvasLayer*> layers;
  OOFCanvasLayer *current_layer;
  guint lineWidth;
  guint32 lineColor, fillColor; // , bgColor;
  double margin;
  void underlay();
public:
  OOFCanvas(bool antialias);
  virtual ~OOFCanvas();
  virtual GtkWidget *widget() const { return canvas; }
  void add_self(PyObject*);
  void attach_self_to_table(PyObject*, guint *l, guint *r, guint *t, guint *b,
			    GtkAttachOptions*, GtkAttachOptions*,
			    guint*, guint*);
  virtual void close();

  bool is_empty() const { return empty; }

  // Zoom routines.
  double get_pixels_per_unit() const { return pixels_per_unit; }
  void set_pixels_per_unit(double);
  void zoom(double);		// scale by the given factor

  // Size of canvas in pixels
  int get_width_in_pixels() const;
  int get_height_in_pixels() const;

  // This calls the GtkCanvas's "set_scroll_region", which restricts
  // the allowed scrolling area to the passed-in rectangle.  It has
  // the side-effect of repositioning the offsets so that as much of
  // the passed-in rectangle as possible is visible.  It has no effect
  // on what portion of the canvas is drawable.
  void set_scrollregion(const CRectangle&);

  CRectangle get_scrollregion() const;
  void set_hadjustment(PyObject*);
  void set_vadjustment(PyObject*);
  Coord world_coord(int, int) const; // convert window to world coordinates

  // The limits of what's actually been drawn so far.
  CRectangle get_bounds() const;

  OOFCanvasLayer *newLayer();
  void deleteLayer(OOFCanvasLayer*);

  void clear();
  void show();

  void set_lineWidth(unsigned int);
  void set_lineColor(guint32*);
  void set_fillColor(guint32*);
  void set_underlay_params(const std::vector<gushort>*, double);
  void set_bgColor_local(const std::vector<gushort>*);
  void set_bgColor(const std::vector<gushort>*);
  void set_margin(double f);
  void set_margin_local(double f);

  void draw_dot(const Coord *dot);
  void draw_triangle(const Coord *triangle, double angle);
  void draw_segment(const GtkCanvasPoints *segment);
  void draw_segments(const std::vector<GtkCanvasPoints*>*);
  void draw_curve(const GtkCanvasPoints *curve);
  void draw_polygon(const GtkCanvasPoints *polygon);
  void fill_polygon(const GtkCanvasPoints *polygon);
  void draw_circle(const Coord&, double);
  void fill_circle(const Coord&, double);
  void draw_image(const StringImage*, int iw, int ih,
		  const Coord&, double w, double h);
  void draw_shaped_image(const StringImage*, int iw, int ih,
			 const Coord&, double w, double h, GdkImlibColor*);


  void set_rubberband(RubberBand*);

//   void set_rubberband(RUBBERBAND_TYPE*);

  // Separate callbacks for mouse events and resize events.
  void set_mouse_callback(PyObject*);
  void set_configure_callback(PyObject *);

  // Callback functions
  static void gtk_destroy(GtkWidget*, gpointer);
  virtual void destroyed();
  static gint mouse_event(GtkCanvasItem*, GdkEvent*, gpointer);
  void mouse_eventCB(GtkCanvasItem*, GdkEvent*);
  static gint configure_event(GtkCanvas*, GtkAllocation*, gpointer);
  void configure_eventCB(GtkAllocation*);
  friend class OOFCanvasLayer;
  friend class RubberBand;
};

class ScrolledOOFCanvas : public OOFCanvas {
protected:
  GtkWidget *scrollwindow;
public:
  ScrolledOOFCanvas(bool antialias);
  virtual ~ScrolledOOFCanvas();
  virtual GtkWidget *widget() const { return scrollwindow; }
  virtual void destroyed();
  static void gtk_destroy(GtkWidget*, gpointer);
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

class OOFCanvasLayer {
protected:
  GtkCanvasItem *grp, *subgrp;
  OOFCanvas *canvas;
public:
  OOFCanvasLayer(OOFCanvas*);
  ~OOFCanvasLayer();
  GtkCanvasGroup *group() const;
  void raise_layer(int);
  void raise_to_top();
  void lower_layer(int);
  void show();
  void hide();
  void move(double, double);
  void clear();
  void make_current();
  void destroy();
};

#endif
