# -*- python -*-
# $RCSfile: parameterwidgets.py,v $
# $Revision: 1.106 $
# $Author: reida $
# $Date: 2004/12/10 18:55:49 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# GTK widgets for inputting Parameter objects.


### Change setValidity(validity) to widgetChanged(validity,
### interactive).  WhoWidget can use this to send its (self,
### interactive) signal.

from SWIG.common import switchboard
from common import debug
from common import enum
from common import guitop
from common import primitives
from common import strfunction
from common import utils
from common.IO import parameter
from common.IO.GUI import oof_mainiteration
from common.IO.GUI import widgetscope
from gtk import *
from types import *
import string

############################

class ParameterWidget:
    def __init__(self, gtk, scope):
        debug.mainthreadTest()
        self.gtk = gtk                  # base of gtk widget heirarchy
        self.scope = scope              # WidgetScope containing this widget
        self._valid = 0
        if scope:
            scope.addWidget(self)
        self.gtk.connect('destroy', self.destroyCB)

    # Widgets that contain other widgets must redefine show().  show()
    # must call the show() function of each of the subwidgets, instead
    # of just calling gtk.show_all().  This is because the subwidgets
    # might have parts that shouldn't be shown in all contexts.
    def show(self):
        self.gtk.show_all()

    # Widget destruction.  Destruction is initiated either when
    # external code calls ParameterWidget.destroy(), or when the gtk
    # widget is destroyed by gtk.  ParameterWidget.cleanUp() can be
    # redefined in derived classes if there's other stuff to be done.
    # The derived classes must then be sure to call the base class
    # cleanUp function.
    def destroy(self):
        debug.mainthreadTest()
        self.gtk.destroy()
    def destroyCB(self, *args):
        self.gtk = None
        self.cleanUp()
    def cleanUp(self):                  # redefine in derived classes
        if self.scope:
            self.scope.removeWidget(self)
            self.scope = None

    # Widget subclasses should call widgetChanged whenever their value
    # changes.  widgetChanged issues a switchboard call when the
    # validity changes.
    def widgetChanged(self, validity, interactive):
        if self._valid and not validity:
            self._valid = 0
            switchboard.notify(('validity', self), 0)
        elif not self._valid and validity:
            self._valid = 1
            switchboard.notify(('validity', self), 1)
        switchboard.notify(self, interactive)
    def isValid(self):
        return self._valid
    def __repr__(self):
        return self.__class__.__name__

class GenericWidget(ParameterWidget):
    def __init__(self, param, scope=None):
#        self.parameter = param
        gtk = GtkEntry()
        gtk.connect('changed', self.changedCB)
        ParameterWidget.__init__(self, gtk, scope)
        self.set_value(param.value)
        self.widgetChanged(self.validValue(param.value), interactive=0)
    def get_value(self):
        debug.mainthreadTest()
        text = self.gtk.get_text().lstrip()
        # Derived classes should redefine get_value() to do something
        # sensible, if possible, when there is no input.  They should
        # return None only if the input is invalid (which may or may
        # not be the same as having no input).
        if text:
            return utils.OOFeval(self.gtk.get_text())
        return None
    def set_value(self, newvalue):
        valuestr = `newvalue`
        self.gtk.set_text(valuestr)
        self.gtk.set_position(0)        # makes MSD visible
    def changedCB(self, gtkobj):
        self.widgetChanged(self.validValue(self.gtk.get_text()), interactive=1)
    def validValue(self, value):
        # Redefine this is in derived classes if necessary.  The
        # argument to validValue is the raw string returned from the
        # widget.  It will be run through 'eval' before being used, so
        # the checking here shouldn't be too strict.  In most cases
        # it's sufficient to check that the string isn't empty.
        # ACTUALLY, validValue is also called from the widget
        # constructor, where the value passed is *not* the raw string.
        # So validValue() must accept either a raw string or an
        # instance of the class that the widget represents.
        # TODO LATER: This is rather ugly, particularly in the
        # subclasses.  This function should either do the oofeval
        # if it gets a string, or be broken up into validStringValue"
        # and "validParameterValue" or something similar.
        return value is not None and string.lstrip(value) != ""

# TODO LATER: Check if this function has any callers.
def makeGenericWidget(self, scope=None):
    return GenericWidget(self, scope)

#########################

# StringParameter widget is almost a GenericWidget.
# It removes leading spaces from the result.

class StringWidget(GenericWidget):
    def __init__(self, param, scope=None):
##        self.param = param
        GenericWidget.__init__(self, param, scope)
    def get_value(self):
        debug.mainthreadTest()
        return self.gtk.get_text().lstrip()
    def set_value(self, value):
        debug.mainthreadTest()
        if type(value) == StringType and string.lstrip(value) != "":
            self.gtk.set_text(value)
            self.widgetChanged(1, interactive=0)
        else:
            self.gtk.set_text("")
            self.widgetChanged(0, interactive=0)

def _StringParameter_makeWidget(self, scope=None):
    return StringWidget(self, scope)

parameter.StringParameter.makeWidget = _StringParameter_makeWidget
        
#########################

from common.IO import automatic

# AutoWidget sets up and handles the basic geometry of automatic
# widgets -- puts in the checkbox, hooks up signals, etc.  Does not
# talk to the parameter, because AutoNameWidgets have special
# requirements for these.
class AutoWidget(ParameterWidget):
    def __init__(self, param, scope=None):
        gtk = GtkHBox()
        ParameterWidget.__init__(self, gtk, scope)
        self.autocheck = GtkCheckButton()
        self.text = GtkEntry()
        self.gtk.add(self.autocheck, expand=0, fill=0)
        self.gtk.add(self.text, expand=1, fill=1)
        self.autocheck.connect("clicked", self.checkCB)
        self.textsignal = self.text.connect('changed', self.entryCB)
        
    def checkCB(self, gtkobj):
        debug.mainthreadTest()
        if self.autocheck.get_active():
            self.text.signal_handler_block(self.textsignal)
            self.text.set_editable(1)
            self.text.set_sensitive(1)
            self.text.set_text("")
            self.text.signal_handler_unblock(self.textsignal)
            self.widgetChanged(0, interactive=1)
        else:
            self.text.signal_handler_block(self.textsignal)
            self.text.set_editable(0)
            self.text.set_sensitive(0)
            self.text.set_text("automatic")
            self.text.signal_handler_unblock(self.textsignal)
            self.widgetChanged(1, interactive=1)

    def entryCB(self, entry):
        self.widgetChanged(self.autocheck.get_active() and
                           self.text.get_text() != "",
                           interactive=1)

    def get_value(self):
        debug.mainthreadTest()
        if not self.autocheck.get_active():
            return automatic.automatic
        return self.text.get_text()
    
class AutoNameWidget(AutoWidget):
    def __init__(self, param, scope=None):
        AutoWidget.__init__(self, param, scope=scope)
        # Avoid querying param.value here, as it will trigger the
        # autoname resolution process if the parameter is an
        # AutomaticNameParameter or ContextualNameParameter.
        if param.automatic():
            self.set_value(automatic.automatic)
            self.widgetChanged(1, interactive=0)
        else:
            self.set_value(param.truevalue)
            self.widgetChanged(param.truevalue not in ("", None), interactive=0)
        guitop.top().tooltips.set_tip(self.autocheck,
                                      'Switch between typed names and automatically generated names.')
        

    def set_value(self, newvalue):
        debug.mainthreadTest()
        if newvalue is automatic.automatic:
            self.autocheck.set_active(0)
            self.checkCB(self.autocheck)
            self.widgetChanged(1, interactive=0)
        elif newvalue is None or newvalue == '':
            self.autocheck.set_active(1)
            self.checkCB(self.autocheck)
            self.text.set_text('')
            self.widgetChanged(0, interactive=0)
        else:
            self.autocheck.set_active(1)
            self.checkCB(self.autocheck)
            self.text.set_text(newvalue)
            self.widgetChanged(1, interactive=0)
        
def _AutoNameParameter_makeWidget(self, scope):
    return AutoNameWidget(self, scope)
parameter.AutomaticNameParameter.makeWidget = _AutoNameParameter_makeWidget

#########################

# Allows a value of "automatic", or an integer.
class AutoNumberWidget(AutoWidget):
    def __init__(self, param, scope=None):
        AutoWidget.__init__(self, param, scope=None)
        self.set_value(param.value)
        self.widgetChanged(1, interactive=0)
        guitop.top().tooltips.set_tip(self.autocheck,
                                      "Switch between automatic and integer.")

    # The AutoWidget get_value returns automatic or a string, or none.
    # If we get a string, evaluate it and return the result.
    def get_value(self):
        v = AutoWidget.get_value(self)
        if v==automatic.automatic:
            return v
        return utils.OOFeval(v)
        
    def set_value(self, newvalue):
        debug.mainthreadTest()
        if newvalue is automatic.automatic:
            self.autocheck.set_active(0)
            self.checkCB(self.autocheck)
            self.widgetChanged(1, interactive=0)
        elif newvalue is None or newvalue == '':
            self.autocheck.set_active(1)
            self.checkCB(self.autocheck)
            self.text.set_text('')
            self.widgetChanged(0, interactive=0)
        else:
            self.autocheck.set_active(1)
            self.checkCB(self.autocheck)
            self.text.set_text(`newvalue`)
            self.text.set_position(0)
            self.widgetChanged(1, interactive=0)

def _AutoNumberWidget_makeWidget(self, scope):
    return AutoNumberWidget(self, scope)

parameter.AutoIntParameter.makeWidget = _AutoNumberWidget_makeWidget

parameter.AutoNumericParameter.makeWidget = _AutoNumberWidget_makeWidget
    

#########################

class BooleanWidget(ParameterWidget):
    def __init__(self, param, scope=None):
        if param.value:
            labelstr = 'true'
        else:
            labelstr = 'false'
        self.label = GtkLabel(labelstr)
        gtk = GtkFrame()
        ParameterWidget.__init__(self, gtk, scope)
        self.button = GtkCheckButton()
        self.gtk.add(self.button)
        self.button.add(self.label)
        self.signal = self.button.connect('clicked', self.buttonCB)
        map = self.button.get_colormap()
        red = map.alloc("red")
        grn = map.alloc("green")
        style = self.button.get_style().copy()
        style.bg[STATE_ACTIVE] = grn
        style.bg[STATE_NORMAL] = red
        self.button.set_style(style)
        self.set_value(param.value)
    def get_value(self):
        debug.mainthreadTest()
        return self.button.get_active()
    def set_value(self, newvalue):
        debug.mainthreadTest()
        self.button.signal_handler_block(self.signal)
        if newvalue:
            self.button.set_active(1)
        else:
            self.button.set_active(0)
        self.button.signal_handler_unblock(self.signal)
        self.widgetChanged(1, interactive=0)
    def buttonCB(self, obj):
        debug.mainthreadTest()
        if self.button.get_active():
            self.label.set_text('true')
        else:
            self.label.set_text('false')
        self.widgetChanged(1, interactive=1)

##class BooleanWidget(ParameterWidget):
##    def __init__(self, param):
##        self.gtk = GtkFrame()
##        hbox = GtkHBox()
##        self.gtk.add(hbox)
##        self.truebutton = GtkRadioButton(label="true")
##        self.falsebutton = GtkRadioButton(label="false", group=self.truebutton)
##        hbox.add(self.truebutton, expand=0, fill=0)
##        hbox.add(self.falsebutton, expand=0, fill=0)
##        map = self.truebutton.get_colormap()
##        tstyle = self.truebutton.get_style().copy()
##        green = map.alloc("green")
##        red = map.alloc("red")
##        yellow = map.alloc("yellow")
##        blue = map.alloc("blue")
##        magenta = map.alloc("magenta")
##        tstyle.bg[STATE_ACTIVE] = green
##        tstyle.bg[STATE_NORMAL] = green
##        tstyle.text[STATE_NORMAL] = yellow
##        tstyle.text[STATE_ACTIVE] = blue
##        tstyle.text[STATE_PRELIGHT] = magenta
##        self.truebutton.set_style(tstyle)
##        fstyle = self.falsebutton.get_style().copy()
##        fstyle.bg[STATE_ACTIVE] = red
##        fstyle.bg[STATE_NORMAL] = red
##        self.falsebutton.set_style(fstyle)
##    def get_value(self):
##        return self.truebutton.get_active()
##    def set_value(self, newvalue):
##        if newvalue:
##            self.truebutton.set_active(1)
##        else:
##            self.falsebutton.set_active(1)

def _Boolean_makeWidget(self, scope):
    return BooleanWidget(self, scope)
parameter.BooleanParameter.makeWidget = _Boolean_makeWidget

##########################

from common.IO.GUI import labelledslider

class IntRangeWidget(ParameterWidget, labelledslider.LabelledSlider):
    def __init__(self, param, scope=None):
        if param.value is not None:
            val = param.value
        else:
            val = param.range[0]
        labelledslider.LabelledSlider.__init__(self, val,
                                               vmin=param.range[0],
                                               vmax=param.range[1],
                                               step=1,
                                               callback=self.sliderCB)
        self.set_digits(0)
        ParameterWidget.__init__(self, self.gtk, scope)
        self.widgetChanged(1, interactive=0) # always valid
    def get_value(self):
        return int(labelledslider.LabelledSlider.get_value(self))
    def sliderCB(self, slider, val):
        self.widgetChanged(1, interactive=1)

def _IntRange_makeWidget(self, scope):
    return IntRangeWidget(self, scope)

parameter.IntRangeParameter.makeWidget = _IntRange_makeWidget

# Alternative implementation using GtkSpinButton...
##class IntRangeWidget(ParameterWidget):
##    def __init__(self, param):
##        adj = GtkAdjustment(value=param.value,
##                            lower=param.range[0], upper=param.range[1],
##                            step_incr=1, page_incr=1)
##        self.gtk = GtkSpinButton(adj, digits=0)
##    def get_value(self):
##        return int(self.gtk.get_value_as_float())
##    def set_value(self, value):
##        self.gtk.set_value(value)


class FloatRangeWidget(ParameterWidget, labelledslider.LabelledSlider):
    def __init__(self, param, scope=None):
        if param.value is not None:
            val = param.value
        else:
            val = param.range[0]
        labelledslider.LabelledSlider.__init__(self, val,
                                               vmin=param.range[0],
                                               vmax=param.range[1],
                                               step=param.range[2],
                                               callback=self.sliderCB)
        ParameterWidget.__init__(self, self.gtk, scope)
        self.widgetChanged(1, interactive=0) # always valid
    ## FloatRangeWidget used LabelledSlider's get_value()
    def sliderCB(self, slider, val):
        self.widgetChanged(1, interactive=1)

def _FloatRange_makeWidget(self, scope=None):
    return FloatRangeWidget(self, scope)
parameter.FloatRangeParameter.makeWidget = _FloatRange_makeWidget

##############################################

class FloatWidget(GenericWidget):
    def get_value(self):
        x = GenericWidget.get_value(self)
        if x is not None:
            return x
        return 0.0
    def validValue(self, string):
        # Any string is a potentially valid value
        return 1
    def set_value(self, newvalue):
        GenericWidget.set_value(self, 1.0*newvalue)

def _FloatParameter_makeWidget(self, scope=None):
    return FloatWidget(self, scope)
parameter.FloatParameter.makeWidget = _FloatParameter_makeWidget

#########################

class IntWidget(GenericWidget):
    def get_value(self):
        x = GenericWidget.get_value(self)
        if x is not None:
            return x
        return 0
    def validValue(self, string):
        # Any string is a potentially valid value
        return 1

def _IntParameter_makeWidget(self, scope=None):
    return IntWidget(self, scope)

parameter.IntParameter.makeWidget = _IntParameter_makeWidget


#######################

# Widgets for things that are mostly generic, but which should return
# "automatic" if no input has been provided.

##class AutoGenericWidget(GenericWidget):
##    def get_value(self):
##        r = GenericWidget.get_value(self)
##        if r is not None:
##            return r
##        return automatic.automatic
##    def validValue(self, value):
##        return 1

##def _makeAutoGenericWidget(self, scope=None):
##    return AutoGenericWidget(self, scope)

##parameter.AutoNumericParameter.makeWidget = _makeAutoGenericWidget

########################

class XYStrFunctionWidget(GenericWidget):
    def get_value(self):
        debug.mainthreadTest()
        return strfunction.XYStrFunction(self.gtk.get_text().lstrip())
    def set_value(self, newvalue):
        debug.mainthreadTest()
        if newvalue is not None:
            self.gtk.set_text(newvalue.funcstr)
        else:
            self.gtk.set_text('')
    def validValue(self, value):
        if type(value) is StringType:
            try:
                fn = strfunction.XYStrFunction(value)
                return 1
            except:
                return 0
        return isinstance(value, strfunction.XYStrFunction)
            
def _XYStrFunctionParameter_makeWidget(self, scope=None):
    return XYStrFunctionWidget(self, scope)
        
strfunction.XYStrFunctionParameter.makeWidget = \
                                              _XYStrFunctionParameter_makeWidget

############################################

class ParameterTable(ParameterWidget, widgetscope.WidgetScope):
    # A table of Parameter Widgets
    def __init__(self, params, scope=None):
        debug.mainthreadTest()
        widgetscope.WidgetScope.__init__(self, scope)
        self.params = params            # list of Parameters
        if self.params:
            gtk = GtkTable(rows=len(params), cols=2)
        else:
            gtk = GtkVBox()
        ParameterWidget.__init__(self, gtk, scope)
        self.widgets = []
        self.sbcallbacks = []           # switchboard callbacks
        self.subscopes = []             # Widgetscopes for ParameterGroups
        self.set_values()
        self.show()
##        self.base_value = None # Original, pre-selection value, in base form.

    def set_values(self):
        self.labelboxes = []
        self.widgets = []
        self.validities = [0]*len(self.params)
        self.destroySubScopes()
        self.makeWidgets()
        self.signal(interactive=0)

    def makeWidgets(self):
        debug.mainthreadTest()
        subscopedict = {}
        for i in range(len(self.params)):
            param = self.params[i]
            if param.group is None:
                scope = self
            else:
                # Get or make a WidgetScope for this ParameterGroup
                try:
                    scope = subscopedict[param.group]
                except KeyError:
                    scope = widgetscope.WidgetScope(self)
                    self.subscopes.append(scope)
                    subscopedict[param.group] = scope
            self.makeSingleWidget(param, i, scope)

    def makeSingleWidget(self, param, tablepos, scope):
        widget = param.makeWidget(scope=scope)
        self.widgets.append(widget)

        self.sbcallbacks += [
            switchboard.requestCallback(('validity', widget), self.vcheck,
                                        tablepos),
            switchboard.requestCallback(widget, self.widgetChangeCB)
            ]
        self.validities[tablepos] = widget.isValid()
        
        label = GtkLabel(param.name + ' =')
        label.set_alignment(1.0, 0.5)
        eventbox = GtkEventBox()    # Allows tooltips. Labels are untippable.
        eventbox.add(label)
        self.labelboxes.append(eventbox)
        if param.tip:
            guitop.top().tooltips.set_tip(eventbox, param.tip)
        self.gtk.attach(eventbox, 0, 1, tablepos, tablepos+1, xpadding=5,
                        xoptions=FILL)
        self.gtk.attach(widget.gtk, 1, 2, tablepos, tablepos+1, xpadding=5,
                        xoptions=EXPAND|FILL, yoptions=0)
    def get_values(self):
        debug.mainthreadTest()
        exceptions = []
        for param, widget in zip(self.params, self.widgets):
            # Get as many values as possible, even if setting some of them
            # causes errors.
            val = widget.get_value()
            try:
                param.value = val
            except parameter.ParameterMismatch, exception:
                exceptions.append(exception)
        if exceptions:
            raise exceptions[0]
    def vcheck(self, widgetnumber, validity):
        # callback for ('validity', widget).  This just stores the
        # validity value.  The subsequent widget changed switchboard
        # call actually broadcasts the new validity by calling
        # widgetChanged.
        self.validities[widgetnumber] = validity
    def widgetChangeCB(self, interactive):
        self.signal(interactive)
    def signal(self, interactive):
        for v in self.validities:
            if not v:
                self.widgetChanged(0, interactive)
                return
        self.widgetChanged(1, interactive)
    def dumpValidity(self):
        debug.fmsg(zip([p.name for p in self.params], self.validities))
    def show(self):
        debug.mainthreadTest()
        for widget in self.widgets:
            widget.show()
        for labelbox in self.labelboxes:
            labelbox.show_all()
        self.gtk.show() # Need to show eventboxes in table?
    def cleanUp(self):
        # Make sure we don't have any circular references...
        map(switchboard.removeCallback, self.sbcallbacks)
        self.params = []
        self.widgets = []
        ParameterWidget.cleanUp(self)
        self.destroyScope()
        self.destroySubScopes()
    def destroySubScopes(self):
        for scope in self.subscopes:
            scope.destroyScope()
        self.subscopes = []


class HierParameterTable(ParameterTable):
    # ParameterTable that takes a hierarchical list of Parameters, and
    # puts them in a table with WidgetScopes defined for each level of
    # the hierarchy.  The hierarchical list is just a list of lists of
    # lists of Parameters, with arbitrarily deep nesting.
    def __init__(self, params, scope=None):
        self.paramhier = params
        ParameterTable.__init__(self, utils.flatten_all(params), scope)
    def makeWidgets(self):
        debug.mainthreadTest()
        tablepos = 0
        self._doMakeWidgets(self, tablepos, self.paramhier)
    def _doMakeWidgets(self, scope, tablepos, phier):
        for obj in phier:
            if isinstance(obj, parameter.ParameterBase):
                self.makeSingleWidget(obj, tablepos, scope)
                tablepos += 1
            else:                       # obj is a hierarchical list of params
                subscope = widgetscope.WidgetScope(scope)
                tablepos = self._doMakeWidgets(subscope, tablepos, obj)
        return tablepos
                
#####################################################################

class ParameterDialog(widgetscope.WidgetScope):
    def __init__(self, *parameters, **kwargs):
        debug.mainthreadTest()
        # A title for the dialog box can be specified by an optional
        # 'title' keyword argument.  A WidgetScope can be specified
        # with a 'scope' keyword.
        try:
            scope = kwargs['scope']
        except KeyError:
            scope = None
        widgetscope.WidgetScope.__init__(self, scope)
        
        self.done = 0
        self.ok_exit = 0
        self.parameters = parameters
        self.dialog = GtkDialog()
        # Experimental call to influence window placement.
##        self.dialog.set_transient_for(guitop.top().gtk)
        self.dialog.connect("destroy", self.cancelCB)
        self.okbutton = GtkButton('OK')
        self.okbutton.connect('clicked', self.okCB)
        self.dialog.action_area.add(self.okbutton)
        cancelbutton = GtkButton('Cancel')
        cancelbutton.connect('clicked', self.cancelCB)
        self.dialog.action_area.add(cancelbutton)

        try:
            title = kwargs['title']
        except KeyError:
            pass
        else:
            self.dialog.set_title(title)
            hbox = GtkHBox()
            self.dialog.vbox.add(hbox, expand=0, fill=0, padding=5)
            hbox.add(GtkLabel(title), expand=1, fill=1, padding=10)

        self.table = ParameterTable(utils.tupify(parameters), scope=self)
        self.sbcallback = switchboard.requestCallback(('validity', self.table),
                                                      self.validityCB)
        self.dialog.vbox.add(self.table.gtk, expand=0, fill=0)
        accelgroup = GtkAccelGroup()
        self.dialog.add_accel_group(accelgroup)
        self.okbutton.add_accelerator('clicked', accelgroup, GDK.Return,
                                 0, ACCEL_VISIBLE)
        self.sensitize()

    def show(self):
        debug.mainthreadTest()
        self.dialog.set_modal(1)
        self.table.show()
        self.dialog.action_area.show_all()
        self.dialog.show()
    def close(self):
        debug.mainthreadTest()
        switchboard.removeCallback(self.sbcallback)
        self.dialog.destroy()
        self.destroyScope()

    def validityCB(self, validity):
        self.sensitize()

    def sensitize(self):
        self.okbutton.set_sensitive(self.table.isValid())

    def hide(self):
        debug.mainthreadTest()
        self.dialog.hide()

    def okCB(self, *args):
        self.ok_exit = 1
        self.done = 1
        try:
            self.table.get_values()     # copies from widgets to Parameters
        except:
            self.ok_exit = None         # something oo occurred
            raise
        
    def cancelCB(self, *args):
        self.done = 1

def getParameters(*params, **kwargs):
    # Given a bunch of Parameters, create a dialog box for setting
    # them.  If the function returns 1, then the parameters have been
    # set and their values can be extracted by the calling routine.
    # kwargs can contain 'title' and 'scope' arguments.
    dialog = ParameterDialog(*params, **kwargs)
    dialog.show()
    while not dialog.done:
        oof_mainiteration.mainiteration()
    result = dialog.ok_exit
    dialog.close()
    return result

def getParameterValues(*params, **kwargs):
    # This version of getParameters extracts and returns the Parameter
    # values, or returns None if they haven't been set. kwargs can
    # contain 'title' and 'scope' arguments.
    if getParameters(*params, **kwargs):
        return [p.value for p in params]

#####################################################################

from common.IO.GUI import chooser

# Widget base class for subclasses of the Enum class.
class EnumWidget(ParameterWidget):
    def __init__(self, enumclass, param, scope=None):
        self.enumclass = enumclass
        nameset = list(self.enumclass.names)
        self.widget = chooser.ChooserWidget(nameset, self.selection,
                                            helpdict=self.enumclass.helpdict)
        if param.value is not None:
            self.set_value(param.value)
        else:
            self.set_value(enumclass(enumclass.names[0]))
        switchboard.requestCallback(enumclass, self.update)
        ParameterWidget.__init__(self, self.widget.gtk, scope)
        self.widgetChanged(len(nameset) > 0, interactive=0)
    def update(self):
        self.widget.update(list(self.enumclass.names), self.enumclass.helpdict)
        self.widgetChanged(len(self.enumclass.names) > 0, interactive=0)
    def selection(self, gtkobj, name):
        self.value = self.enumclass(name)
        self.widgetChanged(validity=1, interactive=1)
    def get_value(self):
        return self.value
    def set_value(self, value):
        self.value = value
        self.widget.set_state(value.name)

    
def _EnumParameter_makeWidget(self, scope=None):
    return EnumWidget(self.enumclass, self, scope)

enum.EnumParameter.makeWidget = _EnumParameter_makeWidget


#######################################################################

# ValueSetParameter is valid if it's a nontrivial string, or if
# it's a positive integer, or if it's a tuple of things that can
# be converted to floats.
class ValueSetParameterWidget(GenericWidget):
    def validValue(self, value):
        if value is None:
            return 0
        if type(value) is StringType:
            if string.lstrip(value)=="":
                return 0
            return 1 # Nontrival strings are OK.
        
        if type(value) is IntType and value>0:
            return 1 # Ints greater than zero are OK.

        if type(value) is TupleType:
            for v in value:
                try:
                    x = float(v)
                except ValueError:
                    return 0
            return 1


def _makeVSPWidget(self, scope=None):
    return ValueSetParameterWidget(self, scope)

parameter.ValueSetParameter.makeWidget = _makeVSPWidget

#######################################################################

class PointWidget(ParameterWidget):
    def __init__(self, param, scope=None):
        hbox = GtkHBox()
        xlabel = GtkLabel("x:")
        xlabel.set_alignment(1.0, 0.5)
        self.xwidget = FloatWidget(parameter.FloatParameter('Tweedledum', 0))
        ylabel = GtkLabel("y:")
        ylabel.set_alignment(1.0, 0.5)
        self.ywidget = FloatWidget(parameter.FloatParameter('Tweedledee', 0))
        hbox.add(xlabel, expand=0, fill=0, padding=2)
        hbox.add(self.xwidget.gtk, expand=1, fill=1)
        hbox.add(ylabel, expand=0, fill=0, padding=2)
        hbox.add(self.ywidget.gtk, expand=1, fill=1)
        self.sbcallbacks=[
            switchboard.requestCallback(self.xwidget, self.widgetChangeCB),
            switchboard.requestCallback(self.ywidget, self.widgetChangeCB)
            ]
        ParameterWidget.__init__(self, hbox, scope)
        self.set_value(param.value)
        self.widgetChanged(self.xwidget.isValid() and self.ywidget.isValid(),
                           interactive=0)
    def set_value(self, point):
        self.xwidget.set_value(point.x)
        self.ywidget.set_value(point.y)
    def get_value(self):
        return primitives.Point(self.xwidget.get_value(),
                                self.ywidget.get_value())
    def widgetChangeCB(self, interactive):
        self.widgetChanged(self.xwidget.isValid() and self.ywidget.isValid(),
                           interactive)
    def cleanUp(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        ParameterWidget.cleanUp(self)

def _PointParameter_makeWidget(self, scope=None):
    return PointWidget(self, scope)

primitives.PointParameter.makeWidget = _PointParameter_makeWidget
 
