# -*- python -*-
# $RCSfile: activeareamodmenu.py,v $
# $Revision: 1.25 $
# $Author: haan $
# $Date: 2004/12/09 22:51:59 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from SWIG.common import switchboard
from common import activeareamod
from common import utils
from common.IO import automatic
from common.IO import mainmenu
from common.IO import oofmenu
from common.IO import parameter
from common.IO import whoville
from common.IO import xmlmenudump
import common.microstructure


def _undo(menuitem, microstructure):
    ms = common.microstructure.getMicrostructure(microstructure)
    activearea = ms.activearea
    activearea.begin_writing()
    activearea.undo()
    activearea.end_writing()
    switchboard.notify("active area modified", None)
    switchboard.notify("redraw")
    
def _redo(menuitem, microstructure):
    ms = common.microstructure.getMicrostructure(microstructure)
    activearea = ms.activearea
    activearea.begin_writing()
    activearea.redo()
    activearea.end_writing()
    switchboard.notify("active area modified", None)
    switchboard.notify("redraw")
                                                    
def _override(menuitem, microstructure, override):
    ms = common.microstructure.getMicrostructure(microstructure)
    ms.activearea.override(override)
    switchboard.notify("active area modified", None)
    switchboard.notify("redraw")

def activeAreaNameResolver(param, startname):
    if param.automatic():
        basename = 'activearea'
    else:
        basename = startname
    msname = param.group['microstructure'].value
    ms = common.microstructure.getMicrostructure(msname)
    return utils.uniqueName(basename, ms.activeAreaNames())

def _store(menuitem, microstructure, name):
    ms = common.microstructure.getMicrostructure(microstructure)
    ms.saveActiveArea(name)
    switchboard.notify('stored active areas changed', name)

def _rename(menuitem, microstructure, oldname, newname):
    ms = common.microstructure.getMicrostructure(microstructure)
    ms.renameActiveArea(oldname, newname)
    switchboard.notify('stored active areas changed', newname)

def _restore(menuitem, microstructure, name):
    ms = common.microstructure.getMicrostructure(microstructure)
    naa = ms.getNamedActiveArea(name)
    ms.activearea.begin_writing()
    naa.activearea.getTimeStamp().increment()
    ms.activearea.pushModification(naa.activearea)
    ms.activearea.end_writing()
    switchboard.notify('active area modified', None)
    switchboard.notify('redraw')

def _delete(menuitem, microstructure, name):
    ms = common.microstructure.getMicrostructure(microstructure)
    ms.deleteActiveArea(name)
    switchboard.notify('stored active areas changed', name)

# Build menu items for active area modifications

aamodmenu = mainmenu.OOF.addItem(oofmenu.OOFMenuItem('ActiveArea', cli_only=1))
                                                                                
def buildActiveAreaModMenu():
    aamodmenu.clearMenu()
                                                                                
    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Undo', callback=_undo, params=[
            whoville.WhoParameter('microstructure',
                                  common.microstructure.microStructures,
                                  tip=parameter.emptyTipString)
            ],
                            help="Undo the latest active area operation.",
        discussion="<para>Go back to the previous state of &active;.</para>"))
                                                                                
    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Redo', callback=_redo, params=[
            whoville.WhoParameter('microstructure',
                                  common.microstructure.microStructures,
                                  tip=parameter.emptyTipString)
            ],
                            help="Redo the latest undone active area modification.",
        discussion="<para>Undoing the <xref linkend='MenuItem:OOF.ActiveArea.Undo'/>.</para>"))

    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Override',
                            params=[
        parameter.BooleanParameter('override', 0, tip="Whether to override or not."),
        whoville.WhoParameter('microstructure',
                              common.microstructure.microStructures,
                              tip=parameter.emptyTipString)
        ],
                            callback=_override,
                            help="Temporarily override the current active area -- activates all area.",
        discussion=xmlmenudump.loadFile('DISCUSSIONS/common/menu-activearea_override.xml')))

    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Store',
                            callback=_store,
                            params=parameter.ParameterGroup(
        whoville.WhoParameter('microstructure',
                              common.microstructure.microStructures,
                              tip=parameter.emptyTipString),
        parameter.AutomaticNameParameter('name', value=automatic.automatic,
                                         resolver=activeAreaNameResolver,
                                         tip = "The name of the active are to be stored")),
                            help="Store the current active area with a unique name.",
                            discussion="<para>If the current &active; is fairly tough to reproduce and it needs to be re-used later, it is not a bad idea to store it! Stored &actives; are listed on the left-hand side of the active area <link linkend='Section:Tasks:ActiveArea'>task page</link>.</para>"))

    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Restore',
                            callback=_restore,
                            params=[
        whoville.WhoParameter('microstructure',
                              common.microstructure.microStructures,
                              tip=parameter.emptyTipString),
        parameter.StringParameter('name', tip="The name of the active area to be restored.")],
                            help="Restore the active area that is stored with the given name.",
                            discussion="<para>Once <link linkend='MenuItem:OOF.ActiveArea.Store'>stored</link>, an active area can be easily restored in the &micro;, which is the whole point of storing in the first place.</para>"))
    
    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Rename',
                            callback=_rename,
                            params=[
        whoville.WhoParameter('microstructure',
                              common.microstructure.microStructures,
                              tip=parameter.emptyTipString),
        parameter.StringParameter('oldname', tip="The name of the stored active area that is selected for renaming."),
        parameter.StringParameter('newname', '',
                                  tip='A new name for the active area.')
        ],
                            help="Rename the stored active area with a new name.",
                            discussion="<para>If you don't like it, change it.</para>"))
    
    aamodmenu.addItem(
        oofmenu.OOFMenuItem('Delete',
                            callback=_delete,
                            params=[
        whoville.WhoParameter('microstructure',
                              common.microstructure.microStructures,
                              tip=parameter.emptyTipString),
        parameter.StringParameter('name', tip="The name of the active area to be deleted.")],
                            help="Delete the stored active that is selected.",
                            discussion="<para><link linkend='MenuItem:OOF.ActiveArea.Store'>Stored</link> active areas can be deleted, too.</para>"))
    
                           
                                                                                
    for registration in activeareamod.ActiveAreaModifier.registry:
        # registration tip string => menuitem help string
        try:
            help = registration.tip
        except AttributeError:
            help = None
        # registration discussion => menuitem discussion
        try:
            discussion = registration.discussion
        except AttributeError:
            discussion = None
        menuitem = aamodmenu.addItem(
            oofmenu.OOFMenuItem(utils.space2underscore(registration.name()),
                                callback=activeareamod.modify,
                                params=[
            whoville.WhoParameter('microstructure',
                                  common.microstructure.microStructures,
                                  tip=parameter.emptyTipString)
            ] + registration.params,
                                help=help,
                                discussion=discussion))
        menuitem.data = registration
                                                                                
buildActiveAreaModMenu()
