// -*- C++ -*-
// $RCSfile: progressbar.C,v $
// $Revision: 1.17 $
// $Author: redwing $
// $Date: 2004/10/28 15:17:35 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include <Python.h>
#include "common/lock.h"
#include "common/tostring.h"
#include "progressbar.h"
#include "threadstate.h"
#include <iostream>
#include <pthread.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include <vector>


std::vector<Progress*> &listOfProgress() {
  static std::vector<Progress*> listPbars;
  return listPbars;
}

Progress *getProgress() {

  // Default ThreadID constructor makes one that identifies the current thread.
  ThreadID currentThread; 

  std::vector<Progress*> &list = listOfProgress();
  for(int i=0; i<list.size(); i++)
    if(list[i]->get_thread_ID() == currentThread)
      {
	//std::cerr << "returning pbar already in list" << std::endl;
	return list[i];
      }
  //std::cerr << "returing new bar" << std::endl;
  return new Progress();
}



bool operator==(Progress &p1, Progress &p2) {
  return p1.get_threadstate() == p2.get_threadstate();
}


Progress::Progress() : bar_msg(" "), 
			       progress(0),
			       stop(0),
			       success(INCOMPLETE)
{

  tState = findThreadState();
  listOfProgress().push_back(this);
}




Progress::~Progress()
{
  remove_from_list(); //if still on list, remove yourself
}

float Progress::get_progress()
{
  KeyHolder thekey(lockobj);
  return progress;  
}

void Progress::set_message(const std::string & msg)
{
  KeyHolder thekey(lockobj);
  //std::cout << "start" << std::endl;
  //std::cout << "locked" << std::endl; 
  bar_msg = msg;
  //  std::cout << "msg" << std::endl;
  // std::cout <<"unlocked" << std::endl;
}

const std::string Progress::get_message() {
  lock();
  std::string temp_bar_messg = bar_msg;
  unlock();
  return temp_bar_messg;
  
}

void Progress::remove_from_list()
{
  std::vector<Progress*> &list = listOfProgress();
  for(std::vector<Progress*>::iterator i=list.begin(); i<list.end(); ++i) {
    if(*i == this)
      {
	//std::cerr << "removing pBar from list" << std::endl;
	list.erase(i);
      }
  }
}

void Progress::cleanup()
{
  //std::cerr << "cleaning up pBar" << std::endl;
  remove_from_list();
  
}
