# -*- python -*-
# $RCSfile: analyzePage.py,v $
# $Revision: 1.22 $
# $Author: reida $
# $Date: 2004/12/13 17:03:05 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common import guitop
from common import labeltree
from common import utils
from common.IO import automatic
from common.IO import output
from common.IO.GUI import destinationparamwidget
from common.IO.GUI import fileselector
from common.IO.GUI import oofGUI
from common.IO.GUI import outputwidget
from common.IO.GUI import regclassfactory
from common.IO.GUI import whowidget
from engine import analysisdomain
from engine import analysissample
from engine.IO import analyze
from engine.IO import analyzemenu
from engine.IO.GUI import sampleregclassfactory
import engine.mesh
import gtk


# A page on which various aspects of the solved mesh can be queried --
# cross-section and statistical outputs will live here, with the
# ability to be put into files, and so forth.

class AnalyzePage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Analysis",
                                 ordering=259,
                                 tip="Query the mesh, examine fields and fluxes.")
        
        mainbox = gtk.GtkVBox()
        self.gtk.add(mainbox)

        hbox = gtk.GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = gtk.GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        self.meshwidget = whowidget.WhoWidget(engine.mesh.meshes,
                                              # callback=self.meshCB,
                                              scope=self)
        label = gtk.GtkLabel("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[0], expand=0, fill=0)
        label = gtk.GtkLabel("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[1], expand=0, fill=0)
        label = gtk.GtkLabel("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[2], expand=0, fill=0)

        outertable = gtk.GtkTable()
        outertable.set_homogeneous(1)

        mainbox.add(outertable)
        
    
        outputframe = gtk.GtkFrame(label="Output")
        output_scroll = gtk.GtkScrolledWindow()
        output_scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        outputframe.add(output_scroll)
        self.output_obj = outputwidget.ScalarOutputParameterWidget(None,
                                                                   scope=self)
        output_scroll.add_with_viewport(self.output_obj.gtk)
        outertable.attach(outputframe,0,1,0,1,
                          xoptions=gtk.EXPAND+gtk.FILL,
                          yoptions=gtk.EXPAND+gtk.FILL)


        operationframe = gtk.GtkFrame(label="Operation")
        op_scroll = gtk.GtkScrolledWindow()
        op_scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.op_obj = regclassfactory.RegisteredClassFactory(
            analyze.DataOperation.registry, scope=self,
            callback = self.newOperationCB)
        operationframe.add(op_scroll)
        op_scroll.add_with_viewport(self.op_obj.gtk)
        
        outertable.attach(operationframe,0,1,1,2,
                          xoptions=gtk.EXPAND+gtk.FILL,
                          yoptions=gtk.EXPAND+gtk.FILL)

        
        domainframe = gtk.GtkFrame(label="Domain")
        dom_scroll = gtk.GtkScrolledWindow()
        dom_scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.domain_obj = regclassfactory.RegisteredClassFactory(
            analysisdomain.Domain.registry, scope=self,
            callback = self.newDomainCB)
        domainframe.add(dom_scroll)
        dom_scroll.add_with_viewport(self.domain_obj.gtk)

        outertable.attach(domainframe, 1,2,0,1,
                          xoptions=gtk.EXPAND+gtk.FILL,
                          yoptions=gtk.EXPAND+gtk.FILL)

        sampleframe = gtk.GtkFrame(label="Sampling")
        sam_scroll = gtk.GtkScrolledWindow()
        sam_scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.sample_obj = sampleregclassfactory.SampleRCF(
            analysissample.SampleSet.registry, scope=self,
            callback = self.newSampleCB)
        sampleframe.add(sam_scroll)
        sam_scroll.add_with_viewport(self.sample_obj.gtk)
        outertable.attach(sampleframe, 1,2,1,2,
                          xoptions=gtk.EXPAND+gtk.FILL,
                          yoptions=gtk.EXPAND+gtk.FILL)

        actionframe = gtk.GtkFrame()
        actionbox = gtk.GtkHBox()
        actionframe.add(actionbox)

        actionbox.add(gtk.GtkLabel("Destination: "), expand=0,fill=0,padding=2)
        self.destwidget = destinationparamwidget.DestinationParamWidget()
        self.destwidget.set_value(automatic.automatic)
        actionbox.add(self.destwidget.gtk, expand=1, fill=1, padding=2)
        
        self.go_button = gtk.GtkButton("Go")
        self.go_button.connect("clicked", self.go_buttonCB)
        guitop.top().tooltip(self.go_button, "Send the output to the destination.")
        actionbox.add(self.go_button, fill=1, expand=1, padding=2)

        mainbox.add(actionframe, expand=0, fill=0)
        
        # Whenever fields or fluxes are defined or undefined on the
        # mesh, we need to update the output object widget, and
        # possibly invalidate the currently-displayed data, once we
        # start displaying data.
        switchboard.requestCallback("field defined", self.new_mesh_data)
        switchboard.requestCallback("equation activated", self.new_mesh_data)

        switchboard.requestCallback(("new who", "Mesh"), self.new_mesh)

        switchboard.requestCallback( ('validity', self.domain_obj),
                                     self.sensitize_widgets)
        switchboard.requestCallback( ('validity', self.op_obj),
                                     self.sensitize_widgets)
        switchboard.requestCallback( ('validity', self.output_obj),
                                     self.sensitize_widgets)
        switchboard.requestCallback( ('validity', self.destwidget),
                                     self.sensitize_widgets)

        # Also, the allowed sampling options may change when the
        # operation or domain change.
        switchboard.requestCallback(self.op_obj, self.update_sampling)
        switchboard.requestCallback(self.domain_obj, self.update_sampling)

        self.update_sampling()
        self.sensitize_widgets()

    def sensitize_widgets(self, *args):
        go_sensitive = 0
        if self.op_obj.isValid() and self.output_obj.isValid() and \
           self.destwidget.isValid() and self.domain_obj.isValid() and \
           self.meshwidget.get_value():
            go_sensitive = 1
        self.go_button.set_sensitive(go_sensitive)
        
    # Switchboard, "field defined" and "equation activated"
    def new_mesh_data(self, meshname, *args):
        if meshname == self.meshwidget.get_value():
            self.output_obj.treeCB()

    # Switchboard, ("new who", "Mesh")
    def new_mesh(self, mesh):
        path = labeltree.makePath(mesh)
        self.meshwidget.set_value(path)

    # Callback from "operation" registered callback.
    def newOperationCB(self, registration):
        self.sensitize_widgets()

    def newDomainCB(self, registration):
        self.sensitize_widgets()

    def newSampleCB(self, registration):
        self.sensitize_widgets()
    
    # When either the domain or the operation change, the allowed
    # samplings can change -- update the widget here.
    def update_sampling(self, *args):
        domain_reg = self.domain_obj.getRegistration()
        op_reg = self.op_obj.getRegistration()

        if domain_reg:
            sample_types = domain_reg.sample_types
        else:
            sample_types = []

        if op_reg:
            directness = op_reg.direct
        else:
            directness = None

        self.sample_obj.set_defaults()
        self.sample_obj.setSampleCriteria(sample_types, directness)

        self.sample_obj.update(analysissample.SampleSet.registry)


    # "Go" button callback -- fill in the basic parameters, and
    # perform the action.
    def go_buttonCB(self, gtkobj):
        op_reg = self.op_obj.get_value()
        regname = op_reg.getRegistration().name()
        menuitem = analyzemenu.ops_menu.getItem(
            utils.space2underscore(regname))

        menuitem.callWithDefaults(mesh=self.meshwidget.get_value(),
                                  data=self.output_obj.get_value(),
                                  domain=self.domain_obj.get_value(),
                                  sampling=self.sample_obj.get_value(),
                                  destination=self.destwidget.get_value())
        

analyzepage = AnalyzePage()
