# -*- python -*-
# $RCSfile: pinnodesGUI.py,v $
# $Revision: 1.19 $
# $Author: haan $
# $Date: 2004/09/15 19:51:02 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from SWIG.common import switchboard
from common import debug
from common import guitop
from common import utils
from common import primitives
from common.IO import mainmenu
from common.IO import reporter
from common.IO.GUI import mousehandler
from common.IO.GUI import toolboxGUI
from common.IO.GUI import regclassfactory
from engine.IO import pinnodes
from engine.IO import pinnodemodifier
from gtk import *

class PinnedNodesToolboxGUI(toolboxGUI.GfxToolbox, mousehandler.MouseHandler):
    def __init__(self, pinnodestoolbox):
        debug.mainthreadTest()

        toolboxGUI.GfxToolbox.__init__(self, "Pin Nodes", pinnodestoolbox)
        mainbox = GtkVBox()
        self.gtk.add(mainbox)

        infoframe = GtkFrame()
        mainbox.add(infoframe, expand=0, fill=0)
        info = GtkLabel("""Click a node to pin it,
Shift click to unpin it,
And control click to toggle.""")
        infoframe.add(info)
            

        self.table = GtkTable(cols=3, rows=5)
        mainbox.add(self.table, expand=0, fill=0)

        label = GtkLabel('Mouse')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 0,2, xpadding=2, xoptions=0)

        label = GtkLabel('x=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 1,2, 0,1, xpadding=5, xoptions=FILL)
        self.xtext = GtkEntry()
        self.xtext.set_editable(0)
        self.table.attach(self.xtext, 2,3, 0,1,
                          xpadding=2, xoptions=EXPAND|FILL)
        guitop.top().tooltips.set_tip(self.xtext, "x position of the mouse")

        label = GtkLabel('y=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 1,2, 1,2, xpadding=2, xoptions=FILL)
        self.ytext = GtkEntry()
        self.ytext.set_editable(0)
        self.table.attach(self.ytext, 2,3, 1,2,
                          xpadding=2, xoptions=EXPAND|FILL)
        guitop.top().tooltips.set_tip(self.ytext, "y position of the mouse")

        self.table.set_row_spacing(1, 5)

        label = GtkLabel("Node")
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 2,4, xpadding=2, xoptions=0)

        label = GtkLabel('x=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 1,2, 2,3, xpadding=2, xoptions=FILL)
        self.nodextext = GtkEntry()
        self.nodextext.set_editable(0)
        self.table.attach(self.nodextext, 2,3, 2,3,
                          xpadding=2, xoptions=EXPAND|FILL)

        label = GtkLabel('y=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 1,2, 3,4, xpadding=2, xoptions=FILL)
        self.nodeytext = GtkEntry()
        self.nodeytext.set_editable(0)
        self.table.attach(self.nodeytext, 2,3, 3,4,
                          xpadding=2, xoptions=EXPAND|FILL)

        self.pintext = GtkLabel()
        self.pintext.set_alignment(0.0, 0.5)
        self.table.attach(self.pintext, 2,3, 4,5,
                          xpadding=5, xoptions=EXPAND|FILL)

        modbox = GtkVBox()
        self.pinModFactory = regclassfactory.RegisteredClassFactory(
            pinnodemodifier.PinNodeModifier.registry, title="Method:")
        modbox.add(self.pinModFactory.gtk, expand=0, fill=0, padding=3)

        bbox1 = GtkHBox()
        self.okbutton = GtkButton("OK")
        self.okbutton.connect("clicked", self.okCB)
        guitop.top().tooltip(self.okbutton, "Pin nodes with the selected method.")
        self.undobutton = GtkButton("Undo")
        self.undobutton.connect("clicked", self.undoCB)
        guitop.top().tooltip(self.undobutton, "Unpin the latest pinned node.")
        self.redobutton = GtkButton("Redo")
        self.redobutton.connect("clicked", self.redoCB)
        guitop.top().tooltip(self.redobutton, "Repin the latest unpinned node.")
        bbox1.add(self.undobutton, expand=0, fill=0, padding=2)
        bbox1.add(self.okbutton, expand=1, fill=1, padding=2)
        bbox1.add(self.redobutton, expand=0, fill=0, padding=2)
        modbox.add(bbox1, expand=0, fill=0, padding=2)

        bbox2 = GtkHBox()
        self.unpinallbutton = GtkButton("UnPinAll")
        self.unpinallbutton.connect("clicked", self.unpinallCB)
        guitop.top().tooltip(self.unpinallbutton, "Unpin all the pinned nodes.")
        self.invertbutton = GtkButton("Invert")
        self.invertbutton.connect("clicked", self.invertCB)
        guitop.top().tooltip(self.invertbutton, "Invert - pin the unpinned and unpin the pinned.")
        bbox2.add(self.unpinallbutton, expand=1, fill=1, padding=8)
        bbox2.add(self.invertbutton, expand=1, fill=1, padding=8)
        modbox.add(bbox2, expand=0, fill=0, padding=2)
        mainbox.pack_end(modbox, expand=0, fill=0)

        self.status = GtkLabel()
        self.status.set_alignment(0.0, 0.5)
        mainbox.pack_end(self.status, expand=0, fill=0, padding=5)

    def update(self, *args):
        debug.mainthreadTest()
        context = self.toolbox.getSkeletonContext()
        if context:
            n = len(context.pinnednodes.retrieve())
        else:
            n = 0
        self.status.set_text("%d node%s pinned." % (n, 's'*(n!=1)))

        self.unpinallbutton.set_sensitive(not not context and n > 0)
        self.undobutton.set_sensitive(not not context and
                                      context.pinnednodes.undoable())
        self.redobutton.set_sensitive(not not context and
                                      context.pinnednodes.redoable())
        self.invertbutton.set_sensitive(not not context)
        self.okbutton.set_sensitive(not not context)
        
    def skelChanged(self, skelcontext):
        if skelcontext is self.toolbox.getSkeletonContext():
            self.update()
            
    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.gfxwindow().setMouseHandler(self)
        self.sbcallbacks = [
            switchboard.requestCallback(('who changed', 'Skeleton'),
                                        self.skelChanged),
            switchboard.requestCallback((self.gfxwindow(), "new nodes pinned"),
                                        self.update)
            
            ]
        self.update()

    def deactivate(self):
        toolboxGUI.GfxToolbox.deactivate(self)
        self.gfxwindow().removeMouseHandler()
        map(switchboard.removeCallback, self.sbcallbacks)

    def showPosition(self, x, y):
        debug.mainthreadTest()
        self.xtext.set_text("%-11.4g" % x)
        self.ytext.set_text("%-11.4g" % y)

    def acceptEvent(self, eventtype):
        return eventtype in ('move', 'up')

    def up(self, x, y, shift, ctrl):
        self.showPosition(x, y)
        # Perhaps shift and ctrl should be handled by the menu
        # commands, as they are in pixel selection.
        if shift:
            self.toolbox.menu.UnPin(x=x, y=y)
        elif ctrl:
            self.toolbox.menu.TogglePin(x=x, y=y)
        else:
            self.toolbox.menu.Pin(x=x, y=y)

    def move(self, x, y, shift, ctrl):
        debug.mainthreadTest()
        self.xtext.set_text("%-11.4g" % x)
        self.ytext.set_text("%-11.4g" % y)
        skel = self.toolbox.getSkeleton()
        if skel:
            node = skel.nearestNode(primitives.Point(x,y))
            if node:
                pos = node.position()
                self.nodextext.set_text("%-11.4g" % pos.x)
                self.nodeytext.set_text("%-11.4g" % pos.y)
                if node.pinned():
                    self.pintext.set_text('pinned')
                else:
                    self.pintext.set_text('not pinned')
                return
        self.nodextext.set_text('')
        self.nodeytext.set_text('')
        self.pintext.set_text('')


    def okCB(self, button):
        modmethod = self.pinModFactory.getRegistration()
        if modmethod is not None:
            self.pinModFactory.set_defaults()
            menuitem = getattr(self.toolbox.menu,
                               utils.space2underscore(modmethod.name()))
            menuitem.callWithDefaults(context=
                                      self.toolbox.getSkeletonContext().path())

    def invertCB(self, button):
        self.toolbox.menu.Invert()
    
    def undoCB(self, button):
        self.toolbox.menu.Undo()

    def unpinallCB(self, button):
        self.toolbox.menu.UnPinAll()

    def redoCB(self, button):
        self.toolbox.menu.Redo()

def _makeGUI(self):
    return PinnedNodesToolboxGUI(self)

pinnodes.PinnedNodesToolbox.makeGUI = _makeGUI
