# -*- python -*-
# $RCSfile: boundarymenu.py,v $
# $Revision: 1.18 $
# $Author: haan $
# $Date: 2004/11/10 19:13:38 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# Builds the boundary menu from the registrations in the
# BoundaryModifier class.


from engine import boundarybuilder
from engine import boundarymodifier
from engine import skeletoncontext
from engine.IO import skeletonmenu
from engine.IO import skeletongroupparams
from common.IO import oofmenu
from common.IO import mainmenu
from common import utils
from common.IO import parameter
from SWIG.common import switchboard
import types



boundarymenu = skeletonmenu.skeletonmenu.addItem(
    mainmenu.OOFMenuItem('Boundary', cli_only=1))

####  Boundary deletion.

def _deleteCB(menuitem, skeleton, boundary):
    skelctxt = skeletoncontext.skeletonContexts[skeleton]
    skelctxt.removeBoundary(boundary)

boundarymenu.addItem(oofmenu.OOFMenuItem(
    "Delete",
    callback=_deleteCB,
    help="Delete a boundary from the skeleton and meshes.",
    params=[boundarybuilder.skeletonparam,
            skeletongroupparams.BoundaryParameter('boundary',
                                                  tip="Boundary to be removed.")
            ]))

### Boundary renaming

def _renameCB(menuitem, skeleton, boundary, name):
    skelctxt = skeletoncontext.skeletonContexts[skeleton]
    skelctxt.renameBoundary(boundary, name)

boundarymenu.addItem(oofmenu.OOFMenuItem(
    "Rename",
    callback=_renameCB,
    help="Rename a boundary in a skeleton and in associated meshes.",
    params=[boundarybuilder.skeletonparam,
            skeletongroupparams.BoundaryParameter('boundary', tip="Boundary to be renamed."),
            parameter.StringParameter('name', tip="New name.")]))

### Boundary construction, very important...

def _constructCB(menuitem, skeleton, name, constructor):
    skelctxt = skeletoncontext.skeletonContexts[skeleton]
    constructor(skelctxt, name)
    
boundarymenu.addItem(oofmenu.OOFMenuItem(
    "Construct",
    callback=_constructCB,
    help="Construct a new boundary in a skeleton and in the associated meshes.",
    params = [boundarybuilder.skeletonparam,
              boundarybuilder.nameparam,
              parameter.RegisteredParameter(
    "constructor",
    boundarybuilder.BoundaryConstructor,
    tip="Which method to use.")
              ]))

### Boundary modification.

def _modifyCB(self, skeleton, boundary, modifier):
    skelctxt = skeletoncontext.skeletonContexts[skeleton]
    modifier(skelctxt, boundary)

boundarymenu.addItem(oofmenu.OOFMenuItem(
    "Modify",
    callback=_modifyCB,
    help="Add or remove nodes or edges from a boundary.",
    params=[boundarybuilder.skeletonparam,
            skeletongroupparams.BoundaryParameter('boundary',
                                                  tip="Boundary to be modified."),
            boundarymodifier.BoundaryModifierParameter('modifier',
                                                       tip=parameter.emptyTipString)
            ] ))
            
