# -*- python -*-
# $RCSfile: movenode.py,v $
# $Revision: 1.23 $
# $Author: haan $
# $Date: 2004/11/17 22:59:54 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from SWIG.common import switchboard
from common import debug
from common import primitives
from common import toolbox
from common import ringbuffer
from common.IO import oofmenu
from common.IO import parameter
from SWIG.common import timestamp
from common import primitives
from engine import skeletondiff

class SelectedNode:
    def __init__(self, node=None):
        self.timestamp = timestamp.TimeStamp()
        self.node_ = node

    def __repr__(self):
        return "MoveNodeSelection(%s)" % self.node

    def set(self, node=None):
        self.node_ = node
        self.timestamp.increment()
        return self

    def node(self):
        return self.node_

    def getTimeStamp(self):
        return self.timestamp

##########################################################################

class MoveNodeToolbox(toolbox.Toolbox):
    def __init__(self, gfxwindow):
        toolbox.Toolbox.__init__(self, 'Move_Nodes', gfxwindow)
        self.whoset = ('Skeleton',)
        self.selectednode = SelectedNode()
        self.allow_illegal=0
        
    def makeMenu(self, menu):
        self.menu = menu
        menu.addItem(oofmenu.OOFMenuItem(
            'MoveNode',
            threadable=oofmenu.THREADABLE,
            callback = self.moveNode,
            params=[primitives.PointParameter('origin', tip=parameter.emptyTipString),
                    primitives.PointParameter('destination', tip=parameter.emptyTipString)],
            help="Move a node to another positon."
            ))
        menu.addItem(oofmenu.OOFMenuItem(
            'SelectNode',
            callback = self.selectNode,
            params=[parameter.FloatParameter('x', 0.0, tip="The x coordinate."),
                    parameter.FloatParameter('y', 0.0, tip="The y coordinate.")],
            help="Select a node to move."
            ))
        menu.addItem(oofmenu.OOFMenuItem(
            'AllowIllegal',
            callback = self.allowIllegal,
            params=[parameter.BooleanParameter('allowed',0, tip=parameter.emptyTipString)],
            help="Indicate if moves can create illegal elements."))

    def activate(self):
        pass
        
    def moveNode(self, menuitem, origin, destination):
        skelcontext = self.gfxwindow().topwho('Skeleton')
        if skelcontext:
            skeleton = skelcontext.getObject().deputyCopy()
            skeleton.activate()
            node = skeleton.nearestNode(origin)
            skelcontext.reserve()
            skelcontext.begin_writing()
            try:
                skeleton.moveNodeTo(node, destination)
                if node.illegal():
                    if self.allow_illegal==1:
                        skeleton.setIllegal()
                    else:
                        node.moveBack()
                elif skeleton.illegal(): # node motion may have rehabilitated
                    skeleton.checkIllegality()
                skelcontext.pushModification(skeleton)
            finally:
                skelcontext.end_writing()
                skelcontext.cancel_reservation()
            skeleton.needsHash()
            switchboard.notify('redraw')

    def selectNode(self, menuitem, x, y):
        point = primitives.Point(x,y)
        context = apply(self.gfxwindow().topwho, self.whoset)
        if context:
            skeleton = context.getObject()
            self.selectednode.set(node=skeleton.nearestNode(point))
            switchboard.notify('redraw')

    def allowIllegal(self, menuitem, allowed):
        self.allow_illegal=allowed
        switchboard.notify(("illegal-move status changed", self), allowed)
        
toolbox.registerToolboxClass(MoveNodeToolbox, ordering=2.6)
