# -*- python -*-
# $RCSfile: profileIO.py,v $
# $Revision: 1.4 $
# $Author: haan $
# $Date: 2004/11/24 20:46:14 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# Save and load named profiles -- these objects live in the
# ProfileManager, and are defined in engine/profile.py.
# Meshes which have boundary conditions which use named profiles
# should autosave the relevant named profiles.


from common import enum
from common import quit
from common.IO import datafile
from common.IO import mainmenu
from common.IO import oofmenu
from common.IO import parameter
from engine import profile
AllProfiles = profile.AllProfiles

savemenu = mainmenu.OOF.File.Save

# We can call the loaddata menu item "Profile" safely, because
# only named profiles can be saved.
profile_data_menu = mainmenu.OOF.LoadData.addItem(
    oofmenu.OOFMenuItem('Profile'))

def _newNamedProfile(menuitem, name, profile):
    # Setting the name does the dictionary insertion and bookkeeping.
    profile.set_name(name)

profile_data_menu.addItem(oofmenu.OOFMenuItem(
    'New',
    callback=_newNamedProfile,
    params=[parameter.StringParameter('name', tip="Name of the profile."),
            parameter.RegisteredParameter('profile', profile.CoreProfile,
                                          tip=parameter.emptyTipString)],
    help="Create a new named profile from a data file.",
    discussion="<para>Load a <link linkend='RegisteredClass:CoreProfile'><classname>CoreProfile</classname></link> with a given name.</para>"))
            

# This function is called from saveprofile, below, and optionally also
# from the mesh, when it has to save profiles.
def writeprofile(datafile, profile_name, profile_object):
    datafile.startCmd(mainmenu.OOF.LoadData.Profile.New)
    datafile.argument('name', profile_name)
    datafile.argument('profile', profile_object)
    datafile.endCmd()
    
def saveprofile(menuitem, filename, mode, format, profile):
    quit.disallow()
    try:
        profile_obj = AllProfiles[profile]
        dfile = datafile.writeDataFile(filename, mode, format)
        writeprofile(dfile, profile, profile_obj)
        dfile.close()
    finally:
        quit.allow()


# This item is "secret" because it's only invoked either from
# a script or from the bounadry condition page's "Save" button.
profile_save = savemenu.addItem(oofmenu.OOFMenuItem(
    'Profile',
    secret=1,
    callback=saveprofile,
    threadable=oofmenu.THREADABLE,
    params = [
    parameter.StringParameter('filename', tip="File name."),
    parameter.StringParameter('mode', tip="'w' to (over)write and 'a' to append."),
    enum.EnumParameter('format', datafile.DataFileFormat, datafile.ASCII,
                       tip="File format."),
    profile.ProfileNameParameter('profile', tip="Profile to save.")],
    help="Save a named profile to a file.",
    discussion="Available file formats are listed in <xref linkend='Table:data_option'/>."))

