// -*- C++ -*-
// $RCSfile: conjugate.h,v $
// $Revision: 1.2 $
// $Author: redwing $
// $Date: 2004/12/07 15:25:51 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef CONJUGATE_H
#define CONJUGATE_H

class Field;
class FieldIndex;
class Equation;

class FieldComponent {
private:
  FieldIndex * f_component;
  Field * field;
public:
  FieldComponent(Field * fld, FieldIndex * fc)
    : field(fld),
      f_component(fc)
  {}
  Field *get_field() {return field;}
  FieldIndex * get_field_component() {return f_component;}
}; // end of FieldComponent class

class EquationComponent {
private:
  FieldIndex * eq_component;
  Equation * equation;
public:
  EquationComponent(Equation * eqn, FieldIndex * eqn_comp)
    : equation(eqn),
      eq_component(eqn_comp)
  {}
  Equation * get_equation() {return equation;}
  FieldIndex * get_equation_component() {return eq_component;}
}; //end of EquationComponent class

class ConjugatePair {
private:
  FieldComponent * fc;
  EquationComponent *eqncomp;
  const std::string _name;
  
  ConjugatePair() 
    : fc(0), eqncomp(0), _name("") {} //prohibited
  
public:
  ConjugatePair(const std::string nm, Equation *eq, FieldIndex *eq_comp, Field * fld, FieldIndex *fld_comp) 
    : _name(nm)
    //      eqncomp(eq, eq_comp),
    //      fc(fld, fld_comp)
  {
    eqncomp = new EquationComponent(eq, eq_comp);
    fc = new FieldComponent(fld, fld_comp);
  }
  Field * get_field() {return fc->get_field();}
  FieldIndex * get_field_component() {return fc->get_field_component();}
  Equation * get_equation() {return eqncomp->get_equation();}
  FieldIndex * get_equation_component() {return eqncomp->get_equation_component();}
  const std::string get_name() {return _name;}
  EquationComponent * get_equation_component_obj() {return eqncomp;}
  FieldComponent * get_field_component_obj() {return fc;}
  ~ConjugatePair() 
  {
    delete eqncomp;
    delete fc;
  }
}; //end ConjugatePair class

#endif //end of CONJUGATE_H
