# -*- python -*-
# $RCSfile: edgeset_.py,v $
# $Revision: 1.11 $
# $Author: reida $
# $Date: 2004/05/24 19:50:34 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

# from SWIG.engine import node
from SWIG.engine.node import FuncNodePtr
from engine.profile import Location

def _EdgeSet___repr__(self):
    outstr = "EdgeSet( "
    for n in self.nodes():
        outstr = outstr + " %s " % n
    outstr = outstr + ")\n"
    return outstr

EdgeSet.__repr__ = _EdgeSet___repr__


# Utility function for building the cached progress data -- makes the
# expensive call into the C++ code to traverse the boundary, measuring
# progress in terms of nodes, node indices, distance traversed, and
# fractional distance traversed.  Caches the results, because it will
# be needed by each boundary condition.
def _EdgeSet_build_caches_(self):
    raw = self.ndlist()
    elist = map(EdgeNodeDistancePtr, raw)
    self.node_cache = map(lambda x: x.node, elist)
    self.location_cache = []
    for e in elist:
        self.location_cache.append(
            Location(e.node.position(), index=e.index,
                     s=e.distance, alpha=e.fraction))
    
EdgeSet.build_caches_ = _EdgeSet_build_caches_

# This is the function you should call to get a Python list of
# objects measuring progress along the boundary.  It calls the
# swigged-and-typemapped C++ "ndlist" function to get a list of
# EdgeNodeDistance pointers, and extracts the node information from it.
def _EdgeSet_nodes(self):
    try:
        return self.node_cache
    except AttributeError:
        self.build_caches_()
        return self.node_cache

EdgeSet.nodes = _EdgeSet_nodes

# Function to retrieve the list of locations from the edgeset.
# Locations are basically nodes, enhanced with extra information about
# how far along in the boundary you are.
def _EdgeSet_locations(self):
    try:
        self.location_cache
        self.node_cache
    except AttributeError:
        self.build_caches_()

    return map( None, self.node_cache, self.location_cache)

EdgeSet.locations = _EdgeSet_locations


# Wrapper to make sure that addEdge calls make both caches stale.
# Note that the caches are built by, among others, the __repr__, which
# is a little unintuitive.
def _EdgeSet_addEdge(self, boundaryedge):
    try:
        del self.node_cache
    except AttributeError:
        pass
    try:
        del self.location_cache
    except AttributeError:
        pass
    self.addEdge_(boundaryedge)


EdgeSetPtr.addEdge = _EdgeSet_addEdge

# Patterned after the FuncNodeIteratorPtr getitem routine in mesh_.py.
def _EdgeSetIterator___getitem__(self, idx):
    if self.end():
        raise IndexError
    ret = self.edge()
    self.increment()
    return ret

EdgeSetIteratorPtr.__getitem__ = _EdgeSetIterator___getitem__
