# -*- python -*-
# $RCSfile: optimize.py,v $
# $Revision: 1.26 $
# $Author: redwing $
# $Date: 2004/11/08 21:16:40 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import debug
from common import primitives
from common import registeredclass
from common.IO import reporter
from common.IO import parameter
from engine import bdycondition
from engine import fiddlenodesbase
from engine import problem
from engine import profile
from engine import skeletonelement
from engine import skeletonmodifier
from engine import mesh
from engine.IO import meshmenu
from SWIG.common.IO import progressbar
from SWIG.common import devel
from SWIG.common import switchboard
from SWIG.engine import equation
from SWIG.engine import field
from SWIG.engine import masterelement
from SWIG.engine import material
from SWIG.engine import preconditioner
from SWIG.engine import solver
from SWIG.engine import solverstepper
from SWIG.engine.property.elasticity.skelstiffness import skelstiffness
from SWIG.engine.property.heatconductivity import heatconductivity
from SWIG.engine.property.meshqualityexpansion import meshqualityexpansion
from SWIG.engine.property.orientation import orientation
import random

IterationManager = fiddlenodesbase.IterationManager


class Optimize(fiddlenodesbase.FiddleNodes, skeletonmodifier.SkeletonModifier):
    def __init__(self, criterion, iteration):
        T = 0
        targets = None
        fiddlenodesbase.FiddleNodes.__init__(self, targets, criterion, T, iteration)
        self.intro = "preparing to optimize skeleton...  "
        self.header = "optimizing skeleton: "
        self.outro = "optimization done "

        ## create properties for material
        self.skeletonMaterial = material.Material('__bone_marrow__')
        stiffness = skelstiffness.Skeleton()
        thermalexpansion = meshqualityexpansion.SkeletonExpansionTensor()
        thermalconductivity = heatconductivity.heatConductivity()
        orientayshon = orientation.orientationReg()
        ## assign properties for material
        self.skeletonMaterial.addProperty(stiffness)
        self.skeletonMaterial.addProperty(thermalexpansion)
        self.skeletonMaterial.addProperty(thermalconductivity)
        self.skeletonMaterial.addProperty(orientayshon)
        
            
        
    def set_fakematerial(self, skelelement, skeleton):
        return None
    def create_mesh(self, context):
        ## context is a skeleton context
        ## setup element types
        edict = {}
        edict[3] = masterelement.getMasterElementList()['T3_3']
        edict[4] = masterelement.getMasterElementList()['Q4_4']
        skel = context.getObject()
        ## returns a Mesh (Who) object
        meshname = context.path() + ":__internal_mesh__"
        femesh = skel.femesh(edict, self.set_fakematerial)
        meshcontext = mesh.meshes.add(meshname, femesh,
                                      skeleton = skel, elementdict = edict)
        elementiterator = femesh.element_iterator()
        while not elementiterator.end():
            elementiterator.element().setExplicitMaterial(self.skeletonMaterial)
            femesh.addExplicitMaterial(self.skeletonMaterial)
            elementiterator.next()
        femesh.set_microstructure(skel.MS)
        return meshcontext
    def delete_mesh(self, mesh):
        mesh.reserve()
        mesh.begin_writing()
        mesh.destroy()
        
    def define_fields(self, femesh): ## femesh is an femesh
        femesh.define_field(field.getCompoundField('Displacement'))
        femesh.define_field(problem.Temperature)
        ## also activate the fields
        femesh.activate_field(field.getCompoundField('Displacement'))
        field.getCompoundField('Displacement').set_in_plane(femesh, 1)
        
        femesh.activate_field(field.getCompoundField('Temperature'))
        field.getCompoundField('Temperature').set_in_plane(femesh, 1)
        ## debug.fmsg("displacement field is activated", femesh.is_active_field(field.getCompoundField('Displacement')))
        
    def activate_equations(self, femesh): ## femesh is an femesh
        femesh.activate_equation(equation.getEquation('Force_Balance'))
        femesh.activate_equation(equation.getEquation('Heat_Eqn'))
        ## debug.fmsg(" force balance equation is activated", femesh.is_active_equation(equation.getEquation('Force_Balance')))
    def set_boundary_conditions(self, mesh):
        ## here, mesh is a Mesh (Who) object
        
        ## left boundary
        self.leftBoundaryCondition = \
                                   bdycondition.DirichletBC(field.getCompoundField('Displacement'),
                                                            'x',
                                                            equation.getEquation('Force_Balance'),
                                                            'x',
                                                            profile.ConstantProfile(0),
                                                            'left'
                                                            )
        self.leftBoundaryCondition.add_to_mesh('left', mesh.path())
        
        ## right boundary
        self.rightBoundaryCondition = \
                                   bdycondition.DirichletBC(field.getCompoundField('Displacement'),
                                                            'x',
                                                            equation.getEquation('Force_Balance'),
                                                            'x',
                                                            profile.ConstantProfile(0),
                                                            'right'
                                                            )
        self.rightBoundaryCondition.add_to_mesh('right', mesh.path())
        
        ## top boundary
        self.topBoundaryCondition = \
                                   bdycondition.DirichletBC(field.getCompoundField('Displacement'),
                                                            'y',
                                                            equation.getEquation('Force_Balance'),
                                                            'y',
                                                            profile.ConstantProfile(0),
                                                            'top'
                                                            )
        self.topBoundaryCondition.add_to_mesh('top', mesh.path())
        
        ## bottom boundary
        self.bottomBoundaryCondition = \
                                   bdycondition.DirichletBC(field.getCompoundField('Displacement'),
                                                            'y',
                                                            equation.getEquation('Force_Balance'),
                                                            'y',
                                                            profile.ConstantProfile(0),
                                                            'bottom'
                                                            )
        self.bottomBoundaryCondition.add_to_mesh('bottom', mesh.path())
    def update_node_positions(self, skeleton, mesh):
        ## mesh is a Mesh Who object
        femesh = mesh.getObject()
        dispField = field.getCompoundField('Displacement')
        nodes = skeleton.nodes
        for node in nodes:
            realnode =femesh.getNode(node.meshindex)
            dx = dispField.value(femesh, realnode, 0)
            dy = dispField.value(femesh, realnode, 1)
            skeleton.moveNodeBy(node, primitives.Point(dx, dy))

                
    def postProcess(self, context):
        
        ## This function should first create a mesh with the custom-made properties,
        ## second, assign "temporary" properties to pixels
        ## and specify BCs and equations.
        ## third, iterate for the solution using the NR solver
        ## if solution is found, accept node moves and redraw skeleton.
        ## Repeat these steps until iteration criteria has been met
        ## or until NR solver fails.
        ## Finally, trash the mesh and clean-up all the temporary objects

        ## create progress bar
        pBar = progressbar.getProgress()
        pBar.set_message(self.header)
        
        ## get skeleton and calculate energy
        skeleton = context.getObject()
        before = skeleton.energyTotal(self.criterion.alpha)
        self.count = 0
        n = self.iteration.iterations

        ## create temporary mesh (Who) object
        mesh = self.create_mesh(context)

        ## define active fields
        self.define_fields(mesh.femesh())
        mesh.changed()
        ## activate relevant equations
        self.activate_equations(mesh.femesh())
        mesh.changed()
        ## mesh.refreshMaterials()
        
        ## iterate for best node positions,
        ## displaying intermediate configurations
        ## "ideally" should always improve for each
        ## step.
        self.set_boundary_conditions(mesh)
        while self.iteration.goodToGo():
            ## assign boundary conditions
        
            self.count += 1
            ## context thread-safe area starts here
            context.begin_writing()
            try:
                debug.fmsg("before coreProcess")
                self.coreProcess(mesh)
                debug.fmsg("after coreProcess")
                ## coreProcess solves a NR problem for an existing mesh
                ## new node positions are transfered to the skeleton at this point
                self.update_node_positions(skeleton, mesh)
                debug.fmsg("after updating nodal positions")
                skeleton.timestamp.increment()
            finally:
                context.end_writing()
            ## context thread-safe area ends here
                switchboard.notify("redraw") ## redraw every sucessful iteration

            debug.fmsg("after issuing redraw switchboard")    
            self.updateIteration() ## update iteration manager machinery
            debug.fmsg("after updating iteration")
            
            ## update progress bar machinery
            if pBar.query_stop():
                pBar.set_failure()
                pBar.set_message("Failed")
                break
            else:
                if self.pbar_type == "continuous":
                    pBar.set_progress(1.0*self.count/n)
                    pBar.set_message("%s%d/%d"
                                     % (self.header, self.count, n))
        debug.fmsg("before switchboard message notifying nodes moved")
        switchboard.notify("skeleton nodes moved", context)
        debug.fmsg("after switchboard message notifying nodes moved")
        ## update progress bar machinery
        if pBar.query_stop(): 
            pBar.set_failure()
            pBar.set_message("Failed")
            return

        ## calculate total energy improvement, if any.
        after = skeleton.energyTotal(self.criterion.alpha)
        if before:
            rate = 100.0*(before-after)/before
        else:
            rate = 0.0
        diffE = after - before
        reporter.report("%s deltaE = %10.4e (%6.3f%%)"
                        % (self.outro, diffE, rate))
        
        self.delete_mesh(mesh)
        debug.fmsg("after deleting mesh")
    def coreProcess(self, mesh, activenodes = None):
        precond = preconditioner.blockPreconditioner(preconditioner.iluPreconditioner())
        ## print precond
        linear_solver = solver.gmresSolver(1000, 200, 1e-3, precond)
        debug.fmsg("after created GMRESSolver")
        newton_solver = solverstepper.NewtonSolver(linear_solver, 1e-8, 5)
        debug.fmsg("after created NR solver")
        newton_solver.apply(mesh)
        debug.fmsg("after applying NR solver")
        
       
            

#################################################
if devel.devel():
    registeredclass.Registration('Optimize',
                                 skeletonmodifier.SkeletonModifier,
                                 Optimize,
                                 ordering=200,
                                 params=[
        parameter.Parameter('criterion', skeletonmodifier.SkelModCriterion,
                            tip = 'Acceptance criterion'),
        parameter.RegisteredParameter('iteration', IterationManager,
                                      tip='Iteration method')
        ],
                                 tip='Move large number of nodes by optimizing a global energy functional (by oofing our way through)'
                                 )
    
