// -*- C++ -*-
// $RCSfile: skelstiffness.C,v $
// $Revision: 1.7 $
// $Author: redwing $
// $Date: 2004/10/14 18:02:39 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include "engine/cskeleton.h"
#include "engine/femesh.h"
#include "engine/field.h"
#include "engine/flux.h"
#include "engine/material.h"
#include "engine/property/elasticity/skelstiffness/skelstiffness.h"
#include "engine/element.h"
#include "engine/matvec.h"

using namespace std;

// Coef's.  The idea is that the shear modulus is going to be 
// mu0 + alpha*(T-t0), i.e. varies with slope alpha relative to 
// reference temperature t0.  In principle, could be arbitrarily
// complex, of course.
SkeletonStiffness::SkeletonStiffness(PyObject *registry,
				     const string &nm, 
				     Cijkl *cijkl)
  : Elasticity(nm, registry),
    cijkl_(*cijkl)
{
  
}

const Cijkl SkeletonStiffness::cijkl(const FEMesh *mesh, const Element *element, 
				    const MasterPosition &x) const {
  
  // std::cerr << "Cijkl SkeletonStiffness::cijkl" <<std::endl;
  const CSkeletonElement * skelel = element->get_skeleton_element();
  // std::cerr << "after getting skeleton element" <<std::endl;
  double energyH = skelel->energyHomogeneity(*mesh->get_microstructure());
  // std::cerr << "after getting energyH" <<std::endl;
  
  Cijkl modulus = cijkl_;
  // Because temperature variation of modulus is defined in terms
  // of lambda and mu, we have to convert to that representation.

  
  // Construct temperature-dependent modulus from lambda and mu.
  modulus(0,0) = modulus(1,1) = modulus(2,2) = cijkl_(0,0)*(1.-1e-3*energyH);

  // 
  return modulus;
}
