// -*- C++ -*-
// $RCSfile: orientation.C,v $
// $Revision: 1.15 $
// $Author: langer $
// $Date: 2004/03/16 16:45:39 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include "engine/property/orientation/orientation.h"
#include "engine/material.h"
#include "engine/matvec.h"
#include "engine/mvmult.h"
#include "common/trace.h"
using namespace std;

OrientationProp::OrientationProp(PyObject *registry, 
			 const string &nm, const EulerAngle &ea)
  : orient(ea),
    Property(nm,registry)
{
//  Trace("OrientationProp::OrientationProp " + name());
}

OrientationProp::OrientationProp(PyObject *registry, 
				 const string &nm, EulerAngle *ea)
  : orient(*ea),
    Property(nm, registry)
{}


// From the comments in eulerangle.C:
// ... The rotation matrix multiplied by a vector gives the
// coordinates of the vector in a coordinate system that has been
// rotated by the Euler angle.


MATRIX_D OrientationProp::rotate(const MATRIX_D &A) const {
  Trace("OrientationProp::rotate");
  MATRIX_D rot = eulerangle().rotation();
  return transform(A, rot); // rot.A.(rot-transpose)
}

VECTOR_D OrientationProp::rotate(const VECTOR_D &x) const {
  return eulerangle().rotation()*x;
}
