# -*- python -*-
# $RCSfile: thermalexpansion_.py,v $
# $Revision: 1.22 $
# $Author: redwing $
# $Date: 2004/10/22 14:57:44 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from SWIG.engine import symmmatrix
from common import utils
from common.IO import parameter
from engine import problem
from engine import propertyregistration

PropertyRegistration = propertyregistration.PropertyRegistration

class TriclinicThermalExpansion(ThermalExpansionProp):
    def __init__(self, registration, name, alpha, T0):
        self.T0 = T0
        ThermalExpansionProp.__init__(self, registration, self, name,
                                 alpha, T0)
    def __repr__(self):
        return "Triclinic(name='%s', alpha=%s, T0 =%f)" % \
               (self.name(),`self.kappa()`, self.T0)


PropertyRegistration(
    'Thermal:ThermalExpansion:Anisotropic:Triclinic', TriclinicThermalExpansion,
    "SWIG.engine.property.thermalexpansion.thermalexpansion",210,
    [symmmatrix.TriclinicRank2TensorParameter('alpha',
                                     symmmatrix.TriclinicRank2Tensor(xx=1.0, yy=1.0, zz=1.0),
                                     tip="Thermal expansion tensor."),
     parameter.FloatParameter('T0', 0.0, tip="Stress free temperature. Sometimes referred to as the deposition temperature.")],
    fields=[problem.Temperature],
    fluxes=[problem.Stress],
    outputs=["Strain"],
    propertyType="ThermalExpansion")

# Property reprs refer to the registered name, not the actual class name.
def _ThermalExpansion___repr__(self):
    return "TriclinicThermalExpansion(name='%s', alpha=%s, t0=%s)" % \
           (self.name(), `self.alpha()`, `self.tzero()`)

ThermalExpansionPropPtr.__repr__ = _ThermalExpansion___repr__


class IsotropicThermalExpansion(ThermalExpansionProp):
    def __init__(self, registration, name, alpha, T0):
        self.alpha = alpha
        self.T0 = T0
        alphaij = symmmatrix.SymmMatrix3(alpha, alpha, alpha, 0., 0., 0.)
        ThermalExpansionProp.__init__(self, registration, self, name,
                                 alphaij, T0)
    def __repr__(self):
        return "Isotropic(name='%s', alpha=%s, T0=%f)" % \
               (self.name(),`self.alpha`, self.T0)

PropertyRegistration(
    'Thermal:ThermalExpansion:Isotropic', IsotropicThermalExpansion,
    "SWIG.engine.property.thermalexpansion.thermalexpansion",200,
    [parameter.FloatParameter('alpha', 1.0,
                                     tip="Thermal expansion coefficient."),
     parameter.FloatParameter('T0', 0.0, tip="Stress free temperature.")],
    fields=[problem.Temperature],
    fluxes=[problem.Stress],
    outputs=["Strain"],
    propertyType="ThermalExpansion")

class MonoclinicThermalExpansion(ThermalExpansionProp):
    def __init__(self, registration, name, alpha, T0):
        self.T0 = T0
        ThermalExpansionProp.__init__(self, registration, self, name,
                                 alpha, T0)
    def __repr__(self):
        return "Monoclinic(name='%s', alpha=%s, T0 =%f)" % \
               (self.name(),`self.kappa()`, self.T0)


PropertyRegistration(
    'Thermal:ThermalExpansion:Anisotropic:Monoclinic', MonoclinicThermalExpansion,
    "SWIG.engine.property.thermalexpansion.thermalexpansion",209,
    [symmmatrix.MonoclinicRank2TensorParameter(
    'alpha',
    symmmatrix.MonoclinicRank2Tensor(xx=1.0, yy=1.0, zz=1.0), tip="Thermal expansion tensor."),
     parameter.FloatParameter('T0', 0.0, tip="Stress free temperature.")],
    fields=[problem.Temperature],
    fluxes=[problem.Stress],
    outputs=["Strain"],
    propertyType="ThermalExpansion")

class OrthorhombicThermalExpansion(ThermalExpansionProp):
    def __init__(self, registration, name, alpha, T0):
        self.T0 = T0
        ThermalExpansionProp.__init__(self, registration, self, name,
                                 alpha, T0)
    def __repr__(self):
        return "Orthorhombic(name='%s', alpha=%s, T0 =%f)" % \
               (self.name(),`self.kappa()`, self.T0)


PropertyRegistration(
    'Thermal:ThermalExpansion:Anisotropic:Orthorhombic', OrthorhombicThermalExpansion,
    "SWIG.engine.property.thermalexpansion.thermalexpansion",208,
    [symmmatrix.OrthorhombicRank2TensorParameter(
    'alpha',
    symmmatrix.OrthorhombicRank2Tensor(xx=1.0, yy=1.0, zz=1.0), tip="Thermal expansion tensor."),
     parameter.FloatParameter('T0', 0.0, tip="Stress free temperature.")],
    fields=[problem.Temperature],
    fluxes=[problem.Stress],
    outputs=["Strain"],
    propertyType="ThermalExpansion")


class TetragonalThermalExpansion(ThermalExpansionProp):
    def __init__(self, registration, name, alpha, T0):
        self.T0 = T0
        ThermalExpansionProp.__init__(self, registration, self, name,
                                 alpha, T0)
    def __repr__(self):
        return "Tetragonal(name='%s', alpha=%s, T0 =%f)" % \
               (self.name(),`self.kappa()`, self.T0)


PropertyRegistration(
    'Thermal:ThermalExpansion:Anisotropic:Tetragonal', TetragonalThermalExpansion,
    "SWIG.engine.property.thermalexpansion.thermalexpansion",207,
    [symmmatrix.TetragonalRank2TensorParameter(
    'alpha',
    symmmatrix.TetragonalRank2Tensor(xx=1.0, zz=0.5), tip="Thermal expansion tensor."),
     parameter.FloatParameter('T0', 0.0, tip="Stress free temperature.")],
    fields=[problem.Temperature],
    fluxes=[problem.Stress],
    outputs=["Strain"],
    propertyType="ThermalExpansion")


class TrigonalThermalExpansion(ThermalExpansionProp):
    def __init__(self, registration, name, alpha, T0):
        self.T0 = T0
        ThermalExpansionProp.__init__(self, registration, self, name,
                                 alpha, T0)
    def __repr__(self):
        return "Trigonal(name='%s', alpha=%s, T0 =%f)" % \
               (self.name(),`self.kappa()`, self.T0)


PropertyRegistration(
    'Thermal:ThermalExpansion:Anisotropic:Trigonal', TrigonalThermalExpansion,
    "SWIG.engine.property.thermalexpansion.thermalexpansion",206,
    [symmmatrix.TrigonalRank2TensorParameter(
    'alpha',
    symmmatrix.TrigonalRank2Tensor(xx=1.0, zz=0.5), tip="Thermal expansion tensor."),
     parameter.FloatParameter('T0', 0.0, tip="Stress free temperature.")],
    fields=[problem.Temperature],
    fluxes=[problem.Stress],
    outputs=["Strain"],
    propertyType="ThermalExpansion")


class HexagonalThermalExpansion(ThermalExpansionProp):
    def __init__(self, registration, name, alpha, T0):
        self.T0 = T0
        ThermalExpansionProp.__init__(self, registration, self, name,
                                 alpha, T0)
    def __repr__(self):
        return "Hexagonal(name='%s', alpha=%s, T0 =%f)" % \
               (self.name(),`self.kappa()`, self.T0)


PropertyRegistration(
    'Thermal:ThermalExpansion:Anisotropic:Hexagonal', HexagonalThermalExpansion,
    "SWIG.engine.property.thermalexpansion.thermalexpansion",205,
    [symmmatrix.HexagonalRank2TensorParameter(
    'alpha',
    symmmatrix.HexagonalRank2Tensor(xx=1.0, zz=0.5), tip="Thermal expansion tensor."),
     parameter.FloatParameter('T0', 0.0, tip="Stress free temperature.")],
    fields=[problem.Temperature],
    fluxes=[problem.Stress],
    outputs=["Strain"],
    propertyType="ThermalExpansion")


class CubicThermalExpansion(ThermalExpansionProp):
    def __init__(self, registration, name, alpha, T0):
        self.T0 = T0
        ThermalExpansionProp.__init__(self, registration, self, name,
                                 alpha, T0)
    def __repr__(self):
        return "Cubic(name='%s', alpha=%s, T0 =%f)" % \
               (self.name(),`self.kappa()`, self.T0)


PropertyRegistration(
    'Thermal:ThermalExpansion:Anisotropic:Cubic', CubicThermalExpansion,
    "SWIG.engine.property.thermalexpansion.thermalexpansion",204,
    [symmmatrix.CubicRank2TensorParameter(
    'alpha',
    symmmatrix.CubicRank2Tensor(xx=1.0), tip="Thermal expansion tensor."),
     parameter.FloatParameter('T0', 0.0, tip="Stress free temperature.")],
    fields=[problem.Temperature],
    fluxes=[problem.Stress],
    outputs=["Strain"],
    propertyType="ThermalExpansion")
